package com.backend.web.ibdtool.controller;

import com.backend.web.ibdtool.dto.*;
import com.backend.web.ibdtool.exceptions.*;
import com.backend.web.ibdtool.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.security.NoSuchAlgorithmException;

@RestController
public class AuthController {

    @Autowired
    private UserService userService;

    @Value("${ibd-tool-link}") private String link;


    /**
     * Login into the system
     * in Postman testing, send a JSON body in the format {"email":"name@domain.com", "password":"password"}
     * @param loginFormDTO
     * @return
     * @throws NoSuchAlgorithmException
     * @throws UserNotFoundException
     */
    @PostMapping("/auth/login")
    public ResponseEntity<EntityDTO> login(@RequestBody LoginFormDTO loginFormDTO)
            throws UserNotFoundException, UserNotEnabledException, RegistrationExpiredException, NoSuchAlgorithmException {

        EntityDTO entityDTO = userService.login(loginFormDTO);

        if (entityDTO == null)
            return ResponseEntity.badRequest().build();

        return ResponseEntity.ok(entityDTO);
    }

    /* Password recover can't be used if password are saved as HASH!
    /*@PostMapping("auth/login/recover-password")
    public ResponseEntity<String> recoverPassword(@RequestBody PasswordRecoverDTO passwordRecoverDTO) throws IncorrectEmailException, UserNotFoundException, UserNotEnabledException {

        userService.passwordRecovery(passwordRecoverDTO);

        this.link += "/login";

        return ResponseEntity.ok("Password recuperata. Accedi: <a href=" + link + ">Link</a>");

    }*/


    /**
     * Change password request to send
     * @param changePasswordRequestDTO
     * @return
     * @throws IncorrectEmailException
     * @throws UserNotFoundException
     * @throws UserNotEnabledException
     * @throws PasswordNotEqualException
     * @throws PasswordNotChanged
     * @throws NoSuchAlgorithmException
     */
    @PostMapping("auth/login/change-password")
    public ResponseEntity<String> changePassword(
            @RequestBody ChangePasswordRequestDTO changePasswordRequestDTO) throws IncorrectEmailException, UserNotFoundException, UserNotEnabledException, PasswordNotEqualException, PasswordNotChanged, NoSuchAlgorithmException {

        userService.changePasswordRequest(changePasswordRequestDTO);

        return ResponseEntity.ok("Richiesta cambio password");
    }


    /**
     * Password recover procedure
     * @param passwordChangeDTO
     * @return
     * @throws IncorrectEmailException
     * @throws UserNotFoundException
     * @throws UserNotEnabledException
     */
    @PostMapping("auth/login/change-password-confirm/{uuid}/{time}")
    public ResponseEntity<String> changePassword(
            @PathVariable String uuid,
            @PathVariable long time,
            @RequestBody PasswordChangeDTO passwordChangeDTO,
            Model model) throws IncorrectEmailException, UserNotFoundException, UserNotEnabledException, PasswordNotEqualException, PasswordNotChanged, NoSuchAlgorithmException {

        model.addAttribute(uuid);
        model.addAttribute(time);

        System.out.println(time);

        boolean ret = userService.confirmPasswordChange(passwordChangeDTO, uuid, time);

        if (!ret) {
            return ResponseEntity.badRequest().body("Sessione scaduta");
        }

        return ResponseEntity.ok("Password cambiata.");
    }


    /**
     * Logout
     */
    @PostMapping("/auth/logout")
    public void logout() {
    }
}