package com.backend.web.ibdtool.controller;

import com.backend.web.ibdtool.dto.*;
import com.backend.web.ibdtool.entity.Pending;
import com.backend.web.ibdtool.exceptions.*;
import com.backend.web.ibdtool.services.QuestionnaireService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;

@RestController
public class QuestionnaireController {

    @Autowired
    private QuestionnaireService questionnaireService;

    /////////////////////////////////////////////////////////
    ///// PATIENT API //////////////////////////////////////
    /////////////////////////////////////////////////////////

    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @GetMapping("private/patient/get-pending/{patientSSN}")
    public ResponseEntity<ArrayList<Pending>> getPendingQuestionnaires(
            @PathVariable String patientSSN,
            Model model) throws UserNotFoundException, QuestionnaireNotFoundException {

        model.addAttribute(patientSSN);

        ArrayList<Pending> pendingQuestionnaires = questionnaireService.getAllPending(patientSSN);

        if (pendingQuestionnaires != null)
            return ResponseEntity.ok(pendingQuestionnaires);

        return ResponseEntity.badRequest().build();
    }

    //// LIST OF QUESTIONNAIRES ////

    // API FOR BALANCING


    // EMERGENZA PER 14/08
    /*@PreAuthorize("hasAnyAuthority('DOTTORE')")
    @GetMapping("private/doctor/questionnaire/global-sending")
    public ResponseEntity<String> globalSending() throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException, QuestionnaireAlreadyCreatedException {
        questionnaireService.globalSending();
        return ResponseEntity.ok("Invio questionari riuscito");
    }*/


    /**
     * Save Clinical SCCAI
     * @param questionnaireClinicalSCCAI_DTO
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('DOTTORE')")
    @PostMapping("private/patient/questionnaire/send/clinical-sccai/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireClinicalSCCAI
    (@RequestBody QuestionnaireClinicalSCCAI_DTO questionnaireClinicalSCCAI_DTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {

        model.addAttribute(uuid);

        questionnaireService.saveQuestionnaireClinicalSCCAI(questionnaireClinicalSCCAI_DTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save EORTC
     * @param questionnaireEORTC_QLQ_DTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/eortc-qlq/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireEORTCQLQ
    (@RequestBody QuestionnaireEORTC_QLQ_DTO questionnaireEORTC_QLQ_DTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {

        model.addAttribute(uuid);

        questionnaireService.saveQuestionnaireEORTC_QLQ(questionnaireEORTC_QLQ_DTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     *
     * @param questionnaireEQ5D5LDTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PostMapping("private/patient/questionnaire/send/eq5d5l/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireIbdDisk
            (@RequestBody QuestionnaireEQ5D5LDTO questionnaireEQ5D5LDTO,
             @PathVariable String uuid,
             Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {

        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireEQ5D5L(questionnaireEQ5D5LDTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save HADS
     * @param questionnaireHADS_DTO
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/hads/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireHADS
    (@RequestBody QuestionnaireHADS_DTO questionnaireHADS_DTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {

        model.addAttribute(uuid);

        questionnaireService.saveQuestionnaireHADS(questionnaireHADS_DTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }


    /**
     * Receive HBI
     * @param questionnaireHBIDTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/hbi/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireHBI
    (@RequestBody QuestionnaireHBIDTO questionnaireHBIDTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireHBI(questionnaireHBIDTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Receive questionnaire and save with service
     * @param questionnaireIbdDiskDTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/ibd-disk/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireIbdDisk
    (@RequestBody QuestionnaireIBDDiskDTO questionnaireIbdDiskDTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireIBDDisk(questionnaireIbdDiskDTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }


    /**
     * Save IPAQ-SF
     * @param questionnaireIPAQSF_DTO
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/ipaq-sf/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireIPAQSF
    (@RequestBody QuestionnaireIPAQSF_DTO questionnaireIPAQSF_DTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireIPAQSF(questionnaireIPAQSF_DTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save MIAH-CD
     * @param questionnaireMIAHCDDto
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/miah-cd/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireMIAHCD
    (@RequestBody QuestionnaireMIAHCD_DTO questionnaireMIAHCDDto,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireMIAHCD(questionnaireMIAHCDDto, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     *
     * @param questionnaireMIAHUC_DTO
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/miah-uc/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireMIAHUC
    (@RequestBody QuestionnaireMIAHUC_DTO questionnaireMIAHUC_DTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireMIAHUC(questionnaireMIAHUC_DTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save MMAS-8
     * @param questionnaireMMAS8_Dto
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/mmas8/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireMMAS8
    (@RequestBody QuestionnaireMMAS8_DTO questionnaireMMAS8_Dto,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);

        System.out.println(questionnaireMMAS8_Dto);

        questionnaireService.saveQuestionnaireMMAS8(questionnaireMMAS8_Dto, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save patient-SCCAI
     * @param questionnairePatientSCCAI_Dto
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/patient-sccai/{uuid}")
    public ResponseEntity<String> receiveQuestionnairePatientSCCAI
    (@RequestBody QuestionnairePatientSCCAI_DTO questionnairePatientSCCAI_Dto,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnairePatientSCCAI(questionnairePatientSCCAI_Dto, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save PHQ9
     * @param questionnairePHQ9Dto
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/phq9/{uuid}")
    public ResponseEntity<String> receiveQuestionnairePHQ9
    (@RequestBody QuestionnairePHQ9_DTO questionnairePHQ9Dto,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnairePHQ9(questionnairePHQ9Dto, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Receive PRISM
     * @param questionnairePRISMDto
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/prism/{uuid}")
    public ResponseEntity<String> receiveQuestionnairePRISM
    (@RequestBody QuestionnairePRISM_DTO questionnairePRISMDto,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnairePRISM(questionnairePRISMDto, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Save PSQI
     * @param questionnairePSQIDto
     * @param uuid
     * @param model
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws InterruptedException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/psqi/{uuid}")
    public ResponseEntity<String> receiveQuestionnairePSQI
    (@RequestBody QuestionnairePSQI_DTO questionnairePSQIDto,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnairePSQI(questionnairePSQIDto, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Receive SIBDQ
     * @param questionnaireSIBDQDTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/sibdq/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireSIBDQ
    (@RequestBody QuestionnaireSIBDQDTO questionnaireSIBDQDTO,
     @PathVariable String uuid,
     Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireSIBDQ(questionnaireSIBDQDTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Receive TSQM
     * @param questionnaireTSQMDTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/tsqm/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireTSQM
            (@RequestBody QuestionnaireTSQMDTO questionnaireTSQMDTO,
             @PathVariable String uuid,
             Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireTSQM(questionnaireTSQMDTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Receive WPAI-UC
     * @param questionnaireWPAIUCDTO
     * @return
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/questionnaire/send/wpai-uc/{uuid}")
    public ResponseEntity<String> receiveQuestionnaireWPAIUC
            (@RequestBody QuestionnaireWPAIUCDTO questionnaireWPAIUCDTO,
             @PathVariable String uuid,
             Model model) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, RequestNotAuthorizedException, InterruptedException {
        model.addAttribute(uuid);
        questionnaireService.saveQuestionnaireWPAIUC(questionnaireWPAIUCDTO, uuid);

        return ResponseEntity.ok("Questionario ricevuto");
    }

    /**
     * Get all questionnaires of the type specified
     * @param type
     * @param doctorID
     * @param patientSSN
     * @param model
     * @return
     * @throws UserNotFoundException
     * @throws QuestionnaireNotFoundException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @GetMapping("private/patient/get-questionnaires/{type}/{doctorID}/{patientSSN}")
    public ResponseEntity<ArrayList<?>> getQuestionnairesPatient
    (@PathVariable String type,
     @PathVariable String doctorID,
     @PathVariable String patientSSN, Model model) throws UserNotFoundException, QuestionnaireNotFoundException, RequestNotAuthorizedException {

        model.addAttribute(type);
        model.addAttribute(doctorID);
        model.addAttribute(patientSSN);

        ArrayList<?> questionnaires = questionnaireService.getAllQuestionnairesByType(doctorID, type, patientSSN);

        if (questionnaires != null) {
            return ResponseEntity.ok(questionnaires);
        }

        return ResponseEntity.badRequest().build();
    }

    /**
     * Get all questionnaires of the type specified
     * @param patientSSN
     * @param model
     * @return
     * @throws UserNotFoundException
     * @throws QuestionnaireNotFoundException
     */
    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @GetMapping("private/patient/get-updated-questionnaires/{patientSSN}")
    public ResponseEntity<HashSet<String>> getUpdatedQuestionnaires
    (@PathVariable String patientSSN, Model model) throws UserNotFoundException, QuestionnaireNotFoundException, RequestNotAuthorizedException {

        model.addAttribute(patientSSN);

        HashSet<String> questionnaires = questionnaireService.getUpdatedQuestionnaires(patientSSN);

        if (questionnaires != null) {
            return ResponseEntity.ok(questionnaires);
        }

        return ResponseEntity.badRequest().build();
    }


    ///////////////////////////////////////////////////////////////
    /// DOCTOR API  //////////////////////////////////////////////
    //////////////////////////////////////////////////////////////

    /**
     * Get all questionnaires of the type specified
     * @param type
     * @param doctorID
     * @param patientSSN
     * @param model
     * @return
     * @throws UserNotFoundException
     * @throws QuestionnaireNotFoundException
     */
    @PreAuthorize("hasAnyAuthority('DOTTORE')")
    @GetMapping("private/doctor/get-questionnaires/{type}/{doctorID}/{patientSSN}")
    public ResponseEntity<ArrayList<?>> getQuestionnaires
            (@PathVariable String type,
             @PathVariable String doctorID,
             @PathVariable String patientSSN, Model model) throws UserNotFoundException, QuestionnaireNotFoundException, RequestNotAuthorizedException {

        model.addAttribute(type);
        model.addAttribute(doctorID);
        model.addAttribute(patientSSN);

        ArrayList<?> questionnaires = questionnaireService.getAllQuestionnairesByType(doctorID, type, patientSSN);

        if (questionnaires != null) {
            return ResponseEntity.ok(questionnaires);
        }

        return ResponseEntity.badRequest().build();
    }

    /**
     * Create and send a questionnaire by type
     * @param type
     * @param doctorID
     * @param patientSSN
     * @param model
     * @return
     * @throws UserNotFoundException
     * @throws QuestionnaireNotFoundException
     */
    @PreAuthorize("hasAnyAuthority('DOTTORE')")
    @PostMapping("private/doctor/create-and-send-questionnaire/{type}/{doctorID}/{patientSSN}")
    public ResponseEntity<String> createAndSendQuestionnaire
    (@PathVariable String doctorID,
     @PathVariable String patientSSN,
     @PathVariable String type,
     Model model) throws IOException, QuestionnaireAlreadyCreatedException, QuestionnaireAlreadyCompiledException, RequestNotAuthorizedException, InterruptedException {

        model.addAttribute(type);
        model.addAttribute(doctorID);
        model.addAttribute(patientSSN);

        boolean response = questionnaireService.createAndSend(type, doctorID, patientSSN);

        if (response)
            return ResponseEntity.ok("Questionnaire created and sent to patient");

        return ResponseEntity.badRequest().build();
    }

    @PreAuthorize("hasAnyAuthority('DOTTORE')")
    @PostMapping("private/doctor/read-questionnaire/{type}/{doctorID}/{patientSSN}")
    public ResponseEntity<String> readQuestionnaire
            (@PathVariable String type,
             @PathVariable String doctorID,
             @PathVariable String patientSSN,
             @RequestBody String date,
             Model model) throws IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException {

        model.addAttribute(type);
        model.addAttribute(doctorID);
        model.addAttribute(patientSSN);

        questionnaireService.readQuestionnaire(date, type, doctorID, patientSSN);

        return ResponseEntity.ok("Questionnaire read");
    }

    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/warn-doctor-for-questionnaire/{type}/{doctorID}/{patientSSN}")
    public ResponseEntity<String> warnDoctorForQuestionnaire
            (@PathVariable String type,
             @PathVariable String doctorID,
             @PathVariable String patientSSN,
             @RequestBody String date,
             Model model) throws IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException {

        model.addAttribute(type);
        model.addAttribute(doctorID);
        model.addAttribute(patientSSN);

        questionnaireService.warnDoctor(date, type, doctorID, patientSSN);

        return ResponseEntity.ok("Doctor warned");
    }

    @PreAuthorize("hasAnyAuthority('DOTTORE')")
    @PostMapping("private/doctor/evaluate-questionnaire/{type}/{doctorID}/{patientSSN}")
    public ResponseEntity<String> evaluateQuestionnaire
            (@PathVariable String type,
             @PathVariable String doctorID,
             @PathVariable String patientSSN,
             @RequestBody EvaluationDTO evaluationDTO,
             Model model) throws IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException {

        model.addAttribute(type);
        model.addAttribute(doctorID);
        model.addAttribute(patientSSN);

        questionnaireService.evaluateQuestionnaire(type, doctorID, patientSSN, evaluationDTO);

        return ResponseEntity.ok("Questionnaire evaluated");
    }

}
