package com.backend.web.ibdtool.controller;

import com.backend.web.ibdtool.dto.PatientDTO;
import com.backend.web.ibdtool.dto.RegistrationDTO;
import com.backend.web.ibdtool.dto.RegistrationFormDTO;
import com.backend.web.ibdtool.exceptions.*;
import com.backend.web.ibdtool.model.Role;
import com.backend.web.ibdtool.model.UserRole;
import com.backend.web.ibdtool.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

@RestController
public class RegistrationController {

    @Autowired
    private UserService userService;

    @Value("${ibd-tool-link}") private String link;

    /**
     * Register a new doctor in the system
     * @param registrationFormDTO
     * @throws UserAlreadyRegisteredException
     * @throws NoSuchAlgorithmException
     * @throws LogicException
     */
    @PostMapping("/public/register")
    public @ResponseBody
    void registerUser(@RequestBody RegistrationFormDTO registrationFormDTO) throws UserAlreadyRegisteredException, NoSuchAlgorithmException, LogicException {

        RegistrationDTO registrationDTO =
                RegistrationDTO.builder()
                        .email(registrationFormDTO.getEmail().toLowerCase())
                        .name(registrationFormDTO.getName())
                        .surname(registrationFormDTO.getSurname())
                        .SSN(registrationFormDTO.getSSN())
                        .birthDate(registrationFormDTO.getBirthDate())
                        .birthPlace(registrationFormDTO.getBirthPlace())
                        .phoneNumber(registrationFormDTO.getPhoneNumber())
                        .password(registrationFormDTO.getPassword())
                        .role(new Role(UserRole.DOTTORE))
                        .build();

        userService.register(registrationDTO);

    }

    @GetMapping("/public/confirm/{uuid}")
    public ResponseEntity<String> confirm(@PathVariable String uuid,
                                                      Model model) throws IOException, NoSuchAlgorithmException {

        model.addAttribute(uuid);
        userService.confirmRegistration(uuid);

        String page =
                "<head>\n" +
                "<meta http-equiv='refresh' content='0; URL="+ link + "/confirmed-account'>\n" +
                "</head>\n" +
                "<body>\n" +
                "</body>";

        return ResponseEntity.ok(page);
    }

    /**
     * Register new patient
     * @param patientDTO
     * @param email
     * @param model
     * @throws UserNotFoundException
     */
    @PreAuthorize("hasAnyAuthority('DOTTORE')")
    @PostMapping("/private/doctor/register-patient/{email}/{category}")
    public @ResponseBody void registerPatient(@RequestBody PatientDTO patientDTO,
                                              @PathVariable String email,   // email of the patient to be registered
                                              @PathVariable String category,
                                              Model model) throws IOException, NoSuchAlgorithmException, RequestNotAuthorizedException, QuestionnaireAlreadyCreatedException, QuestionnaireAlreadyCompiledException {
        model.addAttribute(email);
        model.addAttribute(category);
        userService.registerPatient(patientDTO, category);
    }

}
