package com.backend.web.ibdtool.controller;

import com.backend.web.ibdtool.dto.EditFormDTO;
import com.backend.web.ibdtool.dto.UserUpdateDTO;
import com.backend.web.ibdtool.exceptions.QuestionnaireAlreadyCompiledException;
import com.backend.web.ibdtool.exceptions.QuestionnaireAlreadyCreatedException;
import com.backend.web.ibdtool.exceptions.RequestNotAuthorizedException;
import com.backend.web.ibdtool.exceptions.UserNotFoundException;
import com.backend.web.ibdtool.model.UserModel;
import com.backend.web.ibdtool.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;

@RestController
public class UserController {

    @Autowired
    private UserService userService;

    @PostMapping("private/user/edit-profile")
    public ResponseEntity<UserUpdateDTO> editProfile(@RequestBody EditFormDTO editFormDTO) throws IOException, RequestNotAuthorizedException, InterruptedException {

        UserUpdateDTO userUpdateDTO = userService.editProfile(editFormDTO);

        if (userUpdateDTO == null) {
            return ResponseEntity.badRequest().body(null);
        }

        return ResponseEntity.ok(userUpdateDTO);
    }


    @PostMapping("private/user/keep-alive")
    public ResponseEntity<String> keepAlive(@RequestBody String email) throws UserNotFoundException, RequestNotAuthorizedException {

        String newTok = userService.keepAlive(email);

        if (newTok == null)
            return ResponseEntity.badRequest().body(null);

        return ResponseEntity.ok(newTok);
    }

    @GetMapping("private/user/pull-updates/{email}/{SSN}")
    public ResponseEntity<UserModel> pullUpdates(@PathVariable String email, @PathVariable String SSN, Model model) throws UserNotFoundException, RequestNotAuthorizedException {

        model.addAttribute(email);
        model.addAttribute(SSN);

        UserModel user = userService.pullUpdates(email, SSN);

        return ResponseEntity.ok(user);
    }

    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @GetMapping("private/patient/is-enabled/{email}")
    public ResponseEntity<Boolean> isPatientEnabled(@PathVariable String email, Model model) throws UserNotFoundException, RequestNotAuthorizedException {

        model.addAttribute(email);

        Boolean enabled = userService.isPatientEnabled(email);

        return ResponseEntity.ok(enabled);
    }

    @PreAuthorize("hasAnyAuthority('PAZIENTE')")
    @PostMapping("private/patient/enable-patient/{email}")
    public ResponseEntity<String> enableOrDisablePatient(@RequestBody String confirmation, @PathVariable String email, Model model) throws IOException, RequestNotAuthorizedException, QuestionnaireAlreadyCreatedException, QuestionnaireAlreadyCompiledException {

        model.addAttribute(email);

        boolean response = Boolean.parseBoolean(confirmation);

        if (response)   // ENABLE PATIENT
            userService.enablePatient(email);

        else            // DELETE PATIENT
            userService.removeUser(email);

        return ResponseEntity.ok("Done");
    }
}
