package com.backend.web.ibdtool.entity;

import com.backend.web.ibdtool.model.*;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import lombok.*;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;

@Document(collection = "users")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@JsonAutoDetect
public class UserEntity implements Serializable {

    @Id
    @Indexed
    private String email;

    private String name;
    private String surname;
    private String SSN;
    private String phoneNumber;
    private String password;

    // private HashSet<Role> roles;  PER IL MOMENTO NON SI HA PIU' DI UN RUOLO
    // si possono registrare solo i medici
    private Role role;


    // Abilitazione account
    private String uuid;
    private boolean enabled;
    private long registrationTime;
    private Long passwordChangeRequest;

    // doctorID info
    private String doctorID;

    // Data e luogo di nascita
    private LocalDate birthDate;
    private String birthPlace;

    // SE DOTTORE -> patients != null
    private ArrayList<Patient> patients;

    // SE PAZIENTE -> doctor, questionnaires e bodyStat != null
    private Doctor doctor;
    private BodyStat bodyStat;
    private HashSet<String> questionnaires;

    // PER TUTTI
    private HashSet<String> tokens;
    private HashSet<NotificationModel> notifications;

    // PER PAZIENTE: CATEGORIA (TELEMEDICINA/TRADIZIONALE)
    private String category;

    // PER TUTTI
    private LocalDateTime lastLogin;
    private long usageTime;

}
