package com.backend.web.ibdtool.security;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.backend.web.ibdtool.entity.UserEntity;
import com.backend.web.ibdtool.exceptions.UserNotFoundException;
import com.backend.web.ibdtool.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashSet;

/**
 * The type Authorization filter.
 */

public class AuthorizationFilter extends BasicAuthenticationFilter {
    private JwtProvider jwtProvider;
    private String prefix;
    private String param;


    @Autowired
    private UserService userService;


    @Autowired
    public AuthorizationFilter(AuthenticationManager authenticationManager, JwtProvider jwtProvider, String prefix, String param, UserService userService) {
        super(authenticationManager);
        this.jwtProvider = jwtProvider;
        this.prefix = prefix;
        this.param = param;
        this.userService = userService;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest req, HttpServletResponse res, FilterChain chain)
            throws IOException, ServletException {
        String header = req.getHeader(param);
        if (header == null || !header.startsWith(prefix)) {
            chain.doFilter(req, res);
            return;
        }
        UsernamePasswordAuthenticationToken authentication = this.getAuthentication(header);
        SecurityContextHolder.getContext().setAuthentication(authentication);
        chain.doFilter(req, res);
    }

    private UsernamePasswordAuthenticationToken getAuthentication(String header) throws UserNotFoundException {
        DecodedJWT decoded = jwtProvider.decodeJwt(header);
        //
        String email = decoded.getSubject();
        //
        UserEntity userEntity = userService.getUser(email);
        HashSet<GrantedAuthority> authorities = new HashSet<>();
        authorities.add(userEntity.getRole());
        //authorities.forEach(a -> System.out.println(a.getAuthority()));
        return new UsernamePasswordAuthenticationToken(
                userEntity.getEmail(),
                userEntity.getPassword(),
                authorities);
    }
}
