package com.backend.web.ibdtool.security;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import lombok.extern.slf4j.Slf4j;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * The type Jwt provider.
 */
@Component
@Slf4j
public class JwtProvider {

    @Value("${security.secret}")
    private String secret;

    @Value("${security.prefix}")
    private String prefix;

    public String createJwt(String email) {

        return JWT.create()
                .withSubject(email)
                .withIssuer("ibd-tool")
                .withIssuedAt(DateTime.now().toDate())
                .withClaim(email, "ibd-tool")
                .withExpiresAt(DateTime.now().plusMinutes(20).toDate())
                .sign(Algorithm.HMAC256(secret));
    }

    public DecodedJWT decodeJwt(String jwt) {
        try {
            jwt = jwt.replace(prefix, "").trim();
            return JWT.require(Algorithm.HMAC256(secret)).build().verify(jwt);
        } catch (Exception e) {
            log.error("Invalid JWT", e);
        }
        return null;
    }
}