package com.backend.web.ibdtool.security;

import com.backend.web.ibdtool.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;

/**
 * The type Security config.
 */
@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private JwtProvider jwtProvider;

    @Autowired
    private final UserService userService;

    @Value("${security.prefix}")
    private String prefix;

    @Value("${security.param}")
    private String param;

    public SecurityConfig(UserService userService) {
        this.userService = userService;
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
                .cors().and().csrf().disable();
        http
                .sessionManagement()
                .sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .addFilter(
                        new AuthorizationFilter(authenticationManager(), jwtProvider, prefix, param, userService)
                )
                .authorizeRequests()
                .antMatchers("/public/**").permitAll()
                .and()
                .authorizeRequests()
                .antMatchers("/auth/login").permitAll()
                .and()
                .authorizeRequests()
                .antMatchers("/auth/logout").authenticated()
                .and()
                .authorizeRequests()
                .antMatchers("/private/**")
                .authenticated()
                .and()
                .authorizeRequests()
                .antMatchers("/notification/**")
                .authenticated();
    }
}
