package com.backend.web.ibdtool.services;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.FileUrlResource;
import org.springframework.mail.MailException;
import org.springframework.mail.MailParseException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;

@Component
public class MailService {

    private static final Logger LOG = LogManager.getLogger(MailService.class.getCanonicalName());
    private JavaMailSender javaMailSender;
    
    @Value("${spring.mail.username}") private String emailFrom;
    @Value("${enable-email:true}") private boolean enabled;
    @Value("${manuale-medico}") private String pathDoctor;
    @Value("${manuale-paziente}") private String pathPatient;

    
    public MailService(@Qualifier("getJavaMailSender") JavaMailSender javaMailSender) {
        this.javaMailSender = javaMailSender;
    }

    public void sendMail(String email, String subject, String message) throws MailException {
        LOG.info("Sending mail to " + email + " - [enabled: " + enabled + "]");

        if (enabled) {
            SimpleMailMessage mail = new SimpleMailMessage();
            mail.setTo(email);
            mail.setFrom(emailFrom);
            mail.setSubject(subject);
            mail.setText(message);
            javaMailSender.send(mail);

        }

    }
}