package com.backend.web.ibdtool.services;

import com.backend.web.ibdtool.dto.*;
import com.backend.web.ibdtool.entity.*;
import com.backend.web.ibdtool.exceptions.*;
import com.backend.web.ibdtool.firebase.NotificationService;
import com.backend.web.ibdtool.model.Patient;
import com.backend.web.ibdtool.model.PrismObject;
import com.backend.web.ibdtool.repository.*;
import com.backend.web.ibdtool.scheduling.SchedulerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
public class QuestionnaireService {

    @Autowired
    private NotificationService notificationService;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PendingRepository pendingRepository;

    @Autowired
    private JobRepository jobRepository;

    @Autowired
    private QuestionnaireClinicalSCCAIRepository questionnaireClinicalSCCAIRepository;

    @Autowired
    private QuestionnaireEORTCQLQRepository questionnaireEORTCQLQRepository;

    @Autowired
    private QuestionnaireEQ5D5LRepository questionnaireEQ5D5LRepository;

    @Autowired
    private QuestionnaireHADSRepository questionnaireHADSRepository;

    @Autowired
    private QuestionnaireHBIRepository questionnaireHBIRepository;

    @Autowired
    private QuestionnaireIBDDiskRepository questionnaireIbdDiskRepository;

    @Autowired
    private QuestionnaireIPAQSFRepository questionnaireIPAQSFRepository;

    @Autowired
    private QuestionnaireMIAHCDRepository questionnaireMIAHCDRepository;

    @Autowired
    private QuestionnaireMIAHUCRepository questionnaireMIAHUCRepository;

    @Autowired
    private QuestionnaireMMAS8Repository questionnaireMMAS8Repository;

    @Autowired
    private QuestionnairePatientSCCAIRepository questionnairePatientSCCAIRepository;

    @Autowired
    private QuestionnairePHQ9Repository questionnairePHQ9Repository;

    @Autowired
    private QuestionnairePRISMRepository questionnairePRISMRepository;

    @Autowired
    private QuestionnairePSQIRepository questionnairePSQIRepository;

    @Autowired
    private  QuestionnaireSIBDQRepository questionnaireSIBDQRepository;

    @Autowired
    private QuestionnaireTSQMRepository questionnaireTSQMRepository;

    @Autowired
    private QuestionnaireWPAIUCRepository questionnaireWPAIUCRepository;

    @Autowired
    private MailService mailService;

    @Autowired
    private SchedulerService schedulerService;

    private Authentication authentication;

    @Value("${ibd-tool-link}") private String link;


    /**
     * Save EORTC
     * @param questionnaireEORTC_QLQ_DTO
     * @throws QuestionnaireNotValidException
     * @throws QuestionnaireAlreadyCompiledException
     */
    public void saveQuestionnaireEORTC_QLQ(QuestionnaireEORTC_QLQ_DTO questionnaireEORTC_QLQ_DTO, String uuid) throws QuestionnaireNotValidException, QuestionnaireAlreadyCompiledException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireEORTC_QLQ_DTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireEORTC_QLQ_DTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireEORTC_QLQ_DTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int[] results = questionnaireEORTC_QLQ_DTO.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        if (results.length < 52) {
            throw new QuestionnaireNotValidException();
        }

        int count = 0;
        for (int r : results) {
            if (r == 0)
                count++;
        }

        if (!((count == 2 && results[results.length - 1] == 0 && results[results.length - 2] == 0) || (count == 0)))
            throw new QuestionnaireNotValidException();

        String doctorID = questionnaireEORTC_QLQ_DTO.getDoctorID();
        String patientSSN = questionnaireEORTC_QLQ_DTO.getPatientSSN();
        String type = "EORTC-QLQ";

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireEORTCQLQRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireEORTC_QLQ questionnaireEORTC_QLQ = QuestionnaireEORTC_QLQ.builder()
                .id(id)
                .type(type)
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireEORTCQLQRepository.save(questionnaireEORTC_QLQ);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * save EQ5D5L
     * @param questionnaireEQ5D5LDTO
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws QuestionnaireAlreadyCompiledException
     */
    public void saveQuestionnaireEQ5D5L(QuestionnaireEQ5D5LDTO questionnaireEQ5D5LDTO, String uuid) throws QuestionnaireNotValidException, QuestionnaireAlreadyCompiledException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireEQ5D5LDTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireEQ5D5LDTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();


        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireEQ5D5LDTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        String[] results = questionnaireEQ5D5LDTO.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        int scale = questionnaireEQ5D5LDTO.getScale();

        if (results.length != 5 || scale < 0 || scale > 100)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "EQ5D5L";
        String doctorID = questionnaireEQ5D5LDTO.getDoctorID();
        String patientSSN = questionnaireEQ5D5LDTO.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireEQ5D5LRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireEQ5D5L questionnaireEQ5D5L = QuestionnaireEQ5D5L.builder()
                .id(id)
                .type(type)
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .scale(scale)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireEQ5D5LRepository.save(questionnaireEQ5D5L);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);

    }

    /**
     * save HBI
     * @param questionnaireHBIDTO
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws IOException
     * @throws InterruptedException
     */
    public void saveQuestionnaireHBI(QuestionnaireHBIDTO questionnaireHBIDTO, String uuid) throws QuestionnaireNotValidException, QuestionnaireAlreadyCompiledException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireHBIDTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireHBIDTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireHBIDTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnaireHBIDTO.getFinalScore();
        String[] results = questionnaireHBIDTO.getResults();
        String[] complications = questionnaireHBIDTO.getComplications();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "HBI";

        String doctorID = questionnaireHBIDTO.getDoctorID();
        String patientSSN = questionnaireHBIDTO.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireHBIRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireHBI questionnaireHBI = QuestionnaireHBI.builder()
                .id(id)
                .type("HBI")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .complications(complications)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        Optional<ArrayList<QuestionnaireHBI>> optionalQuestionnaireHBIS = questionnaireHBIRepository.findAllByTypeAndDoctorIDAndPatientSSN(type, doctorID, patientSSN);

        if (optionalQuestionnaireHBIS.isPresent() && optionalQuestionnaireHBIS.get().size() >= 1) {

            ArrayList<QuestionnaireHBI> questionnaireHBIS = optionalQuestionnaireHBIS.get();
            QuestionnaireHBI last = questionnaireHBIS.get(questionnaireHBIS.size() - 1);

            int lastFinalScore = last.getFinalScore();

            if (finalScore > lastFinalScore + 2) {

                String subject = "HBI - Variazione di +2 punti per " + patient.get().getName() + " " + patient.get().getSurname();
                String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                        "dall'ultimo questionario HBI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                        "è risultato un aumento di almeno 2 punti per l'attività di malattia. \n";

                UserEntity doctorEntity = doctor.get();
                this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
                this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");

            } else if (finalScore < lastFinalScore - 2) {
                String subject = "HBI - Variazione di -2 punti per " + patient.get().getName() + " " + patient.get().getSurname();
                String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                        "dall'ultimo questionario HBI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                        "è risultata una diminuzione di almeno 2 punti per l'attività di malattia. \n";

                UserEntity doctorEntity = doctor.get();
                this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
                this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
            }

            String activityLast = "";
            String activityLatest = "";

            if (lastFinalScore < 5) {
                activityLast = "remissione";
            } else if (lastFinalScore >= 5 && lastFinalScore <= 7) {
                activityLast = "attività lieve";
            } else if (lastFinalScore >= 8 && lastFinalScore <= 16) {
                activityLast = "attività moderata";
            } else if (lastFinalScore > 16) {
                activityLast = "attività grave";
            }

            // LATESI
            if (finalScore < 5) {
                activityLatest = "remissione";
            } else if (finalScore >= 5 && finalScore <= 7) {
                activityLatest = "attività lieve";
            } else if (finalScore >= 8 && finalScore <= 16) {
                activityLatest = "attività moderata";
            } else if (finalScore > 16) {
                activityLatest = "attività grave";
            }

            if (!activityLast.equals(activityLatest)) {
                String subject = "HBI - Passaggio di classe di attività per " + patient.get().getName() + " " + patient.get().getSurname();
                String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                        "dall'ultimo questionario HBI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                        "è risultato un passaggio di classe di attività di malattia da \"" + activityLast + "\" a \"" + activityLatest + "\"\n";

                UserEntity doctorEntity = doctor.get();
                this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
                this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
            }


        }

        questionnaireHBIRepository.save(questionnaireHBI);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);

        if (finalScore >= 8) {
            String subject = "HBI - Ripresa attività di malattia per " + patient.get().getName() + " " + patient.get().getSurname();
            String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                    "dall'ultimo questionario HBI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                    "è risultata una ripresa dell'attività di malattia. \n" +
                    "Il risultato del questionario è di " + finalScore + " con diagnosi di ";

            if (finalScore <= 16 && finalScore >= 8) {
                body += "attività moderata.\n";
            } else if (finalScore > 16) {
                body += "attività grave.\n";
            }

            UserEntity doctorEntity = doctor.get();
            this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
            this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
        }

    }

    /**
     * Save IBDDisk
     * @param questionnaireIbdDiskDTO
     * @param uuid
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    public void saveQuestionnaireIBDDisk(QuestionnaireIBDDiskDTO questionnaireIbdDiskDTO, String uuid) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireIbdDiskDTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireIbdDiskDTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireIbdDiskDTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int[] results = questionnaireIbdDiskDTO.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        boolean verify = Arrays.stream(results)
                .anyMatch(r -> r == 0);

        if (verify)
            throw new QuestionnaireNotValidException();


        String type = "IBD-DISK";
        String doctorID = questionnaireIbdDiskDTO.getDoctorID();
        String patientSSN = questionnaireIbdDiskDTO.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireIbdDiskRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireIBDDisk questionnaireIbdDisk = QuestionnaireIBDDisk.builder()
                .id(id)
                .type("IBD-DISK")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireIbdDiskRepository.save(questionnaireIbdDisk);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save SIBDQ
     * @param questionnaireSIBDQDTO
     * @param uuid
     * @throws QuestionnaireNotValidException
     */
    public void saveQuestionnaireSIBDQ(QuestionnaireSIBDQDTO questionnaireSIBDQDTO, String uuid) throws QuestionnaireNotValidException, QuestionnaireAlreadyCompiledException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireSIBDQDTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireSIBDQDTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireSIBDQDTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        String[] results = questionnaireSIBDQDTO.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "SIBDQ";
        String doctorID = questionnaireSIBDQDTO.getDoctorID();
        String patientSSN = questionnaireSIBDQDTO.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireSIBDQRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireSIBDQ questionnaireSIBDQ = QuestionnaireSIBDQ.builder()
                .id(id)
                .type(type)
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireSIBDQRepository.save(questionnaireSIBDQ);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save TSQM
     * @param questionnaireTSQMDTO
     * @param uuid
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     */
    public void saveQuestionnaireTSQM(QuestionnaireTSQMDTO questionnaireTSQMDTO, String uuid) throws QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireTSQMDTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireTSQMDTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireTSQMDTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        String[] results = questionnaireTSQMDTO.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        int count = 0;

        if (results.length != 14)
            throw new QuestionnaireNotValidException();

        if (results[3].equals("No (in questo caso, passi alla domanda n.9)")) {
            for (int i = 4; i < 8; ++i) {
                if (results[i] != null) {
                    throw new QuestionnaireNotValidException();
                }
                else {
                    count++;
                }
            }
            if (count != 4)
                throw new QuestionnaireNotValidException();
        }



        String type = "TSQM";
        String doctorID = questionnaireTSQMDTO.getDoctorID();
        String patientSSN = questionnaireTSQMDTO.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireTSQMRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireTSQM questionnaireTSQM = QuestionnaireTSQM.builder()
                .id(id)
                .type(type)
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireTSQMRepository.save(questionnaireTSQM);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save WPAI-UC
     * @param questionnaireWPAIUCDTO
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws QuestionnaireAlreadyCompiledException
     */
    public void saveQuestionnaireWPAIUC(QuestionnaireWPAIUCDTO questionnaireWPAIUCDTO, String uuid) throws QuestionnaireNotValidException, QuestionnaireAlreadyCompiledException, QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        if (questionnaireWPAIUCDTO == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireWPAIUCDTO.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireWPAIUCDTO.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        String[] results = questionnaireWPAIUCDTO.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        int count = 0;
        if (results[0].equals("NO")) {
            for (String r : results) {
                if (r == null) {
                    count++;
                }
            }
            if (count != 4)
                throw new QuestionnaireNotValidException();
        }
        else {
            if (results[3].equals("0")) {
                if (results[4] != null)
                    throw new QuestionnaireNotValidException();
            }

            else {
                for (String r : results) {
                    if (r == null) {
                        throw new QuestionnaireNotValidException();
                    }
                }
            }
        }

        String type = "WPAI-UC";
        String doctorID = questionnaireWPAIUCDTO.getDoctorID();
        String patientSSN = questionnaireWPAIUCDTO.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireWPAIUCRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireWPAIUC questionnaireWPAIUC = QuestionnaireWPAIUC.builder()
                .id(id)
                .type(type)
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireWPAIUCRepository.save(questionnaireWPAIUC);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save clinical SCCAI
     * @param questionnaireClinicalSCCAI_dto
     * @param uuid
     * @throws IOException
     * @throws InterruptedException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws QuestionnaireNotValidException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     */
    public void saveQuestionnaireClinicalSCCAI(QuestionnaireClinicalSCCAI_DTO questionnaireClinicalSCCAI_dto, String uuid) throws IOException, InterruptedException, QuestionnaireAlreadyCompiledException, QuestionnaireNotValidException, RequestNotAuthorizedException, QuestionnaireNotFoundException {
        if (questionnaireClinicalSCCAI_dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireClinicalSCCAI_dto.getPatientSSN());
        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireClinicalSCCAI_dto.getDoctorID());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(doctor.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnaireClinicalSCCAI_dto.getFinalScore();
        String[] results = questionnaireClinicalSCCAI_dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "CLINICAL-SCCAI";

        String doctorID = questionnaireClinicalSCCAI_dto.getDoctorID();
        String patientSSN = questionnaireClinicalSCCAI_dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireClinicalSCCAIRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireClinicalSCCAI questionnaireClinicalSCCAI = QuestionnaireClinicalSCCAI.builder()
                .id(id)
                .type("CLINICAL-SCCAI")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireClinicalSCCAIRepository.save(questionnaireClinicalSCCAI);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save HADS in repo
     * @param questionnaireHADS_dto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnaireHADS(QuestionnaireHADS_DTO questionnaireHADS_dto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnaireHADS_dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireHADS_dto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireHADS_dto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnaireHADS_dto.getFinalScore();
        String[] results = questionnaireHADS_dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "HADS";

        String doctorID = questionnaireHADS_dto.getDoctorID();
        String patientSSN = questionnaireHADS_dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireHADSRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireHADS questionnaireHADS = QuestionnaireHADS.builder()
                .id(id)
                .type("HADS")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireHADSRepository.save(questionnaireHADS);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save IPAQ-SF into repo
     * @param questionnaireIPAQSF_dto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnaireIPAQSF(QuestionnaireIPAQSF_DTO questionnaireIPAQSF_dto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnaireIPAQSF_dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireIPAQSF_dto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireIPAQSF_dto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        String diagnosis = questionnaireIPAQSF_dto.getDiagnosis();
        String[] results = questionnaireIPAQSF_dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (int i = 0; i < results.length; i++) {
            if (i == 0 || i == 2 || i == 4) {
                if (results[i].equals("0")) {
                    if (results[i + 1] != null) {
                        throw new QuestionnaireNotValidException();
                    }
                } else {
                    if (results[i] == null) {
                        throw new QuestionnaireNotValidException();
                    }
                }
            } else if (i != 1 && i != 3 && i != 5) {
                if (results[i] == null) {
                    throw new QuestionnaireNotValidException();
                }
            }
        }

        String type = "IPAQ-SF";

        String doctorID = questionnaireIPAQSF_dto.getDoctorID();
        String patientSSN = questionnaireIPAQSF_dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireIPAQSFRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireIPAQSF questionnaireIPAQSF = QuestionnaireIPAQSF.builder()
                .id(id)
                .type("IPAQ-SF")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .diagnosis(diagnosis)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireIPAQSFRepository.save(questionnaireIPAQSF);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save MIAH-CD to repo
     * @param questionnaireMIAHCDDto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnaireMIAHCD(QuestionnaireMIAHCD_DTO questionnaireMIAHCDDto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnaireMIAHCDDto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireMIAHCDDto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireMIAHCDDto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnaireMIAHCDDto.getFinalScore();
        String[] results = questionnaireMIAHCDDto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "MIAH-CD";

        String doctorID = questionnaireMIAHCDDto.getDoctorID();
        String patientSSN = questionnaireMIAHCDDto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireMIAHCDRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireMIAHCD questionnaireMIAHCD = QuestionnaireMIAHCD.builder()
                .id(id)
                .type("MIAH-CD")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireMIAHCDRepository.save(questionnaireMIAHCD);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save MIAH-UC into repo
     * @param questionnaireMIAHUC_dto
     * @param uuid
     * @throws IOException
     * @throws InterruptedException
     * @throws QuestionnaireNotValidException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     */
    public void saveQuestionnaireMIAHUC(QuestionnaireMIAHUC_DTO questionnaireMIAHUC_dto, String uuid) throws IOException, InterruptedException, QuestionnaireNotValidException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException {
        if (questionnaireMIAHUC_dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireMIAHUC_dto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireMIAHUC_dto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnaireMIAHUC_dto.getFinalScore();
        String[] results = questionnaireMIAHUC_dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "MIAH-UC";

        String doctorID = questionnaireMIAHUC_dto.getDoctorID();
        String patientSSN = questionnaireMIAHUC_dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireMIAHUCRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireMIAHUC questionnaireMIAHUC = QuestionnaireMIAHUC.builder()
                .id(id)
                .type("MIAH-UC")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireMIAHUCRepository.save(questionnaireMIAHUC);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save MMAS-8 to repo
     * @param questionnaireMMAS8_dto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnaireMMAS8(QuestionnaireMMAS8_DTO questionnaireMMAS8_dto, String uuid) throws QuestionnaireNotValidException, IOException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException, RequestNotAuthorizedException {
        if (questionnaireMMAS8_dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnaireMMAS8_dto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnaireMMAS8_dto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnaireMMAS8_dto.getFinalScore();
        String[] results = questionnaireMMAS8_dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "MMAS8";

        String doctorID = questionnaireMMAS8_dto.getDoctorID();
        String patientSSN = questionnaireMMAS8_dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnaireMMAS8Repository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnaireMMAS8 questionnaireMMAS8 = QuestionnaireMMAS8.builder()
                .id(id)
                .type("MMAS8")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnaireMMAS8Repository.save(questionnaireMMAS8);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);

        if (finalScore <= 8) {
            String subject = "MMAS-8 - Segnalazione per " + patient.get().getName() + " " + patient.get().getSurname();
            String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                    "dall'ultimo questionario MMAS-8 compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                    "è risultato un punteggio di " + finalScore + "con classe di aderenza: ";

            if (finalScore >= 6) {
                body += "mediamente aderente\n";
            } else if (finalScore < 6) {
                body += "poco aderente\n";
            }

            UserEntity doctorEntity = doctor.get();
            this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
            this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
        }
    }

    /**
     * Save patient-SCCAI to repo
     * @param questionnairePatientSCCAI_dto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnairePatientSCCAI(QuestionnairePatientSCCAI_DTO questionnairePatientSCCAI_dto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnairePatientSCCAI_dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnairePatientSCCAI_dto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnairePatientSCCAI_dto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnairePatientSCCAI_dto.getFinalScore();
        String[] results = questionnairePatientSCCAI_dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "PATIENT-SCCAI";

        String doctorID = questionnairePatientSCCAI_dto.getDoctorID();
        String patientSSN = questionnairePatientSCCAI_dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnairePatientSCCAIRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnairePatientSCCAI questionnairePatientSCCAI = QuestionnairePatientSCCAI.builder()
                .id(id)
                .type("PATIENT-SCCAI")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        Optional<ArrayList<QuestionnairePatientSCCAI>> optionalQuestionnairePatientSCCAIS = questionnairePatientSCCAIRepository.findAllByTypeAndDoctorIDAndPatientSSN(type, doctorID, patientSSN);

        if (optionalQuestionnairePatientSCCAIS.isPresent() && optionalQuestionnairePatientSCCAIS.get().size() >= 1) {

            ArrayList<QuestionnairePatientSCCAI> questionnairePatientSCCAIS = optionalQuestionnairePatientSCCAIS.get();
            QuestionnairePatientSCCAI last = questionnairePatientSCCAIS.get(questionnairePatientSCCAIS.size() - 1);

            int lastFinalScore = last.getFinalScore();

            if (finalScore > lastFinalScore + 2) {

                String subject = "PATIENT-SCCAI - Variazione di +2 punti per " + patient.get().getName() + " " + patient.get().getSurname();
                String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                        "dall'ultimo questionario PATIENT-SCCAI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                        "è risultato un aumento di almeno 2 punti per l'attività di malattia. \n";

                UserEntity doctorEntity = doctor.get();
                this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
                this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");

            } else if (finalScore < lastFinalScore - 2) {
                String subject = "PATIENT-SCCAI - Variazione di -2 punti per " + patient.get().getName() + " " + patient.get().getSurname();
                String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                        "dall'ultimo questionario PATIENT-SCCAI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                        "è risultata una diminuzione di almeno 2 punti per l'attività di malattia. \n";

                UserEntity doctorEntity = doctor.get();
                this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
                this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
            }

            String activityLast = "";
            String activityLatest = "";

            if (lastFinalScore < 5) {
                activityLast = "remissione";
            } else {
                activityLast = "recidiva";
            }

            // LATESI
            if (finalScore < 5) {
                activityLatest = "remissione";
            } else {
                activityLatest = "recidiva";
            }

            if (!activityLast.equals(activityLatest)) {
                String subject = "PATIENT-SCCAI - Passaggio di classe di attività per " + patient.get().getName() + " " + patient.get().getSurname();
                String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                        "dall'ultimo questionario PATIENT-SCCAI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                        "è risultato un passaggio di classe di attività di malattia da \"" + activityLast + "\" a \"" + activityLatest + "\"\n";

                UserEntity doctorEntity = doctor.get();
                this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
                this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
            }


        }

        questionnairePatientSCCAIRepository.save(questionnairePatientSCCAI);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);

        if (finalScore >= 5) {
            String subject = "Patient-SCCAI - Ripresa attività di malattia per " + patient.get().getName() + " " + patient.get().getSurname();
            String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                    "dall'ultimo questionario Patient-SCCAI compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                    "è risultata una ripresa dell'attività di malattia. \n" +
                    "Il risultato del questionario è di " + finalScore + " con diagnosi di recidiva";

            UserEntity doctorEntity = doctor.get();
            this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
            this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
        }
    }

    /**
     * Save PHQ-9 into repo
     * @param questionnairePHQ9Dto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnairePHQ9(QuestionnairePHQ9_DTO questionnairePHQ9Dto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnairePHQ9Dto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnairePHQ9Dto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnairePHQ9Dto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        int finalScore = questionnairePHQ9Dto.getFinalScore();
        String[] results = questionnairePHQ9Dto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (String r : results) {
            if (r == null)
                throw new QuestionnaireNotValidException();
        }

        String type = "PHQ9";

        String doctorID = questionnairePHQ9Dto.getDoctorID();
        String patientSSN = questionnairePHQ9Dto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnairePHQ9Repository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnairePHQ9 questionnairePHQ9 = QuestionnairePHQ9.builder()
                .id(id)
                .type("PHQ9")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .finalScore(finalScore)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnairePHQ9Repository.save(questionnairePHQ9);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);

        if (finalScore >= 5) {
            String subject = "PHQ-9 - Segnalazione depressione per " + patient.get().getName() + " " + patient.get().getSurname();
            String body = "Gentile " + doctor.get().getName() + " " + doctor.get().getSurname() + "\n" +
                    "dall'ultimo questionario PHQ-9 compilato dal paziente " + patient.get().getName() + " " + patient.get().getSurname() + " (codice fiscale: " + patientSSN + ") "  +
                    "è risultato uno stato di depressione con punteggio " + finalScore + "\n";

            UserEntity doctorEntity = doctor.get();
            this.notificationService.sendNotificationToUser(doctorEntity, subject, body);
            this.mailService.sendMail(doctorEntity.getEmail(), subject, body + "IBD Tool - Ospedale Mauriziano");
        }
    }

    /**
     * Save PRISM questionnaire
     * @param questionnairePRISMDto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnairePRISM(QuestionnairePRISM_DTO questionnairePRISMDto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnairePRISMDto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnairePRISMDto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnairePRISMDto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        PrismObject self = questionnairePRISMDto.getSelf();
        PrismObject disease = questionnairePRISMDto.getDisease();
        String distance = questionnairePRISMDto.getDistance();

        if (self == null || disease == null)
            throw new QuestionnaireNotValidException();

        String type = "PRISM";

        String doctorID = questionnairePRISMDto.getDoctorID();
        String patientSSN = questionnairePRISMDto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnairePRISMRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnairePRISM questionnairePRISM = QuestionnairePRISM.builder()
                .id(id)
                .type("PRISM")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .self(self)
                .disease(disease)
                .distance(distance)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnairePRISMRepository.save(questionnairePRISM);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Save PSQI into repo
     * @param questionnairePSQIDto
     * @param uuid
     * @throws QuestionnaireNotValidException
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     * @throws QuestionnaireAlreadyCompiledException
     * @throws InterruptedException
     */
    public void saveQuestionnairePSQI(QuestionnairePSQI_DTO questionnairePSQIDto, String uuid) throws QuestionnaireNotValidException, IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException, QuestionnaireAlreadyCompiledException, InterruptedException {
        if (questionnairePSQIDto == null)
            throw new QuestionnaireNotValidException();

        ////////// SECURITY ///////////
        Optional<UserEntity> patient = userRepository.findBySSN(questionnairePSQIDto.getPatientSSN());

        // check if both doctor and patient are ok and Principal is patient

        if (patient.isEmpty())
            throw new UserNotFoundException();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.get().getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<UserEntity> doctor = userRepository.findByDoctorID(questionnairePSQIDto.getDoctorID());

        if (doctor.isEmpty())
            throw new UserNotFoundException();

        Optional<Pending> optionalPending = pendingRepository.findByUuid(uuid);

        if (optionalPending.isEmpty())
            throw new QuestionnaireNotFoundException();

        Pending pending = optionalPending.get();

        ////////////////////////

        String[] results = questionnairePSQIDto.getResults();

        if (results == null)
            throw new QuestionnaireNotValidException();

        for (int i = 0; i < results.length; ++i) {
            if (i != 14 && i != 15) {
                if (results[i] == null) {
                    throw new QuestionnaireNotValidException();
                }
            }
            if (i == 13) {
                if (results[i] != null && !results[i].equals("No")) {
                    if (results[i + 1] == null || results[i + 2] == null) {
                        throw new QuestionnaireNotValidException();
                    }
                }
            }
        }

        String type = "PSQI";

        String doctorID = questionnairePSQIDto.getDoctorID();
        String patientSSN = questionnairePSQIDto.getPatientSSN();

        String id =
                pending.getDate().format(DateTimeFormatter.ofPattern("dd/MM/yyyy")) + "-" +
                        type + "-" +
                        doctorID + "-" +
                        patientSSN;

        if (questionnairePSQIRepository.findById(id).isPresent())
            throw new QuestionnaireAlreadyCompiledException();

        QuestionnairePSQI questionnairePSQI = QuestionnairePSQI.builder()
                .id(id)
                .type("PSQI")
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .compiled(true)
                .date(pending.getDate())
                .results(results)
                .read(false)
                .evaluation(false)
                .warning(false)
                .build();

        questionnairePSQIRepository.save(questionnairePSQI);
        pendingRepository.delete(pending);
        this.notifyDoctor(doctor.get(), patient.get(), type);
    }

    /**
     * Push notification to Doctor
     * @param doctor
     * @param patient
     * @param type
     */
    public void notifyDoctor(UserEntity doctor, UserEntity patient, String type) throws IOException, InterruptedException {

        String subject = "Nuovo questionario " + type + " compilato da " + patient.getName() + " " + patient.getSurname() + "!";
        String body = "Gentile " + doctor.getName() + " " + doctor.getSurname() + ",\n" +
                "il paziente " + patient.getName() + " " + patient.getSurname() + " (codice fiscale: " + patient.getSSN() + ") ha compilato un nuovo questionario " + type.toUpperCase() + "\n" + "" +
                "Accedi alla tua pagina personale per visualizzarlo.\n" +
                "Clicca qui: " + link + "/patient-questionnaires-show/" + patient.getSSN() + "/" + type.toLowerCase();

        if (type.toUpperCase().equals("CLINICAL-SCCAI")) {
            subject = "Nuovo questionario " + type + " compilato relativo a " + patient.getName() + " " + patient.getSurname() + "!";
            body =  "Hai appena compilato un questionario " + type.toUpperCase() + " relativo al paziente " + patient.getName() + " " + patient.getSurname() + " (codice fiscale: " + patient.getSSN() + ") \n" +
                    "\nClicca qui: " + link + "/patient-questionnaires-show/" + patient.getSSN() + "/" + type.toLowerCase();

        }

        this.notificationService.sendNotificationToUser(doctor, subject, body);
    }



    //////////////////////////////////////////////////////////////////////////////////////////////////

    //// DOCTOR API ////
    /**
     * Get all questionnaires after frontend dispatch request
     * @param doctorID
     * @param type
     * @param patientSSN
     * @return
     * @throws QuestionnaireNotFoundException
     * @throws UserNotFoundException
     */
    public ArrayList<?> getAllQuestionnairesByType(String doctorID, String type, String patientSSN) throws QuestionnaireNotFoundException, UserNotFoundException, RequestNotAuthorizedException {

        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);
        Optional<UserEntity> optionalDoctor = userRepository.findByDoctorID(doctorID);

        if (optionalDoctor.isEmpty())
            throw new UserNotFoundException("Doctor not found!");

        if (optionalPatient.isEmpty())
            throw new UserNotFoundException("Patient not found!");


        switch (type) {
            case "CLINICAL-SCCAI":
                Optional<ArrayList<QuestionnaireClinicalSCCAI>> clinicalSCCAIArrayList = questionnaireClinicalSCCAIRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (clinicalSCCAIArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return clinicalSCCAIArrayList.get();
            case "EORTC-QLQ":
                Optional<ArrayList<QuestionnaireEORTC_QLQ>> eortcQlqArrayList = questionnaireEORTCQLQRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (eortcQlqArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return eortcQlqArrayList.get();
            case "EQ5D5L":
                Optional<ArrayList<QuestionnaireEQ5D5L>> eq5D5LArrayList = questionnaireEQ5D5LRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (eq5D5LArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return eq5D5LArrayList.get();
            case "HADS":
                Optional<ArrayList<QuestionnaireHADS>> hadsArrayList = questionnaireHADSRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (hadsArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return hadsArrayList.get();
            case "HBI":
                Optional<ArrayList<QuestionnaireHBI>> hbiArrayList = questionnaireHBIRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (hbiArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return hbiArrayList.get();
            case "IBD-DISK":
                Optional<ArrayList<QuestionnaireIBDDisk>> ibdDiskArrayList = questionnaireIbdDiskRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (ibdDiskArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return ibdDiskArrayList.get();
            case "IPAQ-SF":
                Optional<ArrayList<QuestionnaireIPAQSF>> ipaqsfArrayList = questionnaireIPAQSFRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (ipaqsfArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return ipaqsfArrayList.get();
            case "MIAH-CD":
                Optional<ArrayList<QuestionnaireMIAHCD>> miahcdArrayList = questionnaireMIAHCDRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (miahcdArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return miahcdArrayList.get();
            case "MIAH-UC":
                Optional<ArrayList<QuestionnaireMIAHUC>> miahucArrayList = questionnaireMIAHUCRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (miahucArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return miahucArrayList.get();
            case "MMAS8":
                Optional<ArrayList<QuestionnaireMMAS8>> mmas8ArrayList = questionnaireMMAS8Repository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (mmas8ArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return mmas8ArrayList.get();
            case "PATIENT-SCCAI":
                Optional<ArrayList<QuestionnairePatientSCCAI>> patientSCCAIArrayList = questionnairePatientSCCAIRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (patientSCCAIArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return patientSCCAIArrayList.get();
            case "PHQ9":
                Optional<ArrayList<QuestionnairePHQ9>> phq9ArrayList = questionnairePHQ9Repository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (phq9ArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return phq9ArrayList.get();
            case "PRISM":
                Optional<ArrayList<QuestionnairePRISM>> prismArrayList = questionnairePRISMRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (prismArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return prismArrayList.get();
            case "PSQI":
                Optional<ArrayList<QuestionnairePSQI>> psqiArrayList = questionnairePSQIRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (psqiArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return psqiArrayList.get();
            case "SIBDQ":
                Optional<ArrayList<QuestionnaireSIBDQ>> sibdqArrayList = questionnaireSIBDQRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (sibdqArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return sibdqArrayList.get();
            case "TSQM":
                Optional<ArrayList<QuestionnaireTSQM>> tsqmArrayList = questionnaireTSQMRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (tsqmArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return tsqmArrayList.get();
            case "WPAI-UC":
                Optional<ArrayList<QuestionnaireWPAIUC>> wpaiucArrayList = questionnaireWPAIUCRepository.findAllByTypeAndPatientSSN(type, patientSSN);
                if (wpaiucArrayList.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else
                    return wpaiucArrayList.get();

            default:
                return null;
        }

    }

    /**
     * set questionnaires for patient
     * @param doctorID
     * @param patientSSN
     * @param questionnaires
     */
    public void setQuestionnaires(String doctorID, String patientSSN, String[] questionnaires) throws QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);

        if (optionalPatient.isEmpty())
            throw new UserNotFoundException();

        UserEntity patient = optionalPatient.get();

        Optional<UserEntity> optionalDoctor = userRepository.findByDoctorID(patient.getDoctorID());

        if (optionalDoctor.isEmpty())
            throw new UserNotFoundException();

        UserEntity doctor = optionalDoctor.get();

        if (questionnaires.length == 0)
            throw new QuestionnaireNotFoundException();

        ArrayList<Patient> patients = doctor.getPatients();

        Patient patientForDoctor = null;
        for (Patient p : patients) {
            if (p.getSSN().equals(patientSSN)) {
                patientForDoctor = p;
            }
        }

        assert patientForDoctor != null;
        patients.remove(patientForDoctor);

        HashSet<String> questionnairesList = patientForDoctor.getQuestionnaires();
        questionnairesList.addAll(Arrays.asList(questionnaires));

        HashSet<String> patientQuestionnaires = patient.getQuestionnaires();

        patientQuestionnaires.addAll(questionnairesList);   // it's a set, if an element already exists, won't be added

        patient.setQuestionnaires(patientQuestionnaires);


        patients.add(patientForDoctor);
        doctor.setPatients(patients);
        //userRepository.deleteByEmail(doctor.getEmail());
        //userRepository.deleteByEmail(patient.getEmail());

        userRepository.save(doctor);
        userRepository.save(patient);

        String body = "Gentile " + patient.getName() + " " + patient.getSurname() + "\n" +
                "sono state aggiunte nuove tipologie di questionari per il tuo account.\n";

        String subject = "IBD Tool - Aggiunta tipologia di questionario";

        this.notificationService.sendNotificationToUser(patient, subject, body);
    }

    /**
     * Remove questionnaire for patient
     * @param doctorID
     * @param patientSSN
     * @param questionnaire
     * @throws UserNotFoundException
     * @throws QuestionnaireNotFoundException
     */
    public void removeQuestionnaire(String doctorID, String patientSSN, String questionnaire) throws QuestionnaireNotFoundException, RequestNotAuthorizedException, IOException, InterruptedException {

        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);

        if (optionalPatient.isEmpty())
            throw new UserNotFoundException();

        UserEntity patient = optionalPatient.get();

        Optional<UserEntity> optionalDoctor = userRepository.findByDoctorID(patient.getDoctorID());

        if (optionalDoctor.isEmpty())
            throw new UserNotFoundException();

        UserEntity doctor = optionalDoctor.get();

        if (questionnaire == null)
            throw new QuestionnaireNotFoundException();

        ArrayList<Patient> patients = doctor.getPatients();

        Patient patientForDoctor = null;
        for (Patient p : patients) {
            if (p.getSSN().equals(patientSSN)) {
                patientForDoctor = p;
            }
        }

        assert patientForDoctor != null;
        patients.remove(patientForDoctor);

        HashSet<String> questionnairesList = patientForDoctor.getQuestionnaires();
        questionnairesList.remove(questionnaire);

        HashSet<String> patientQuestionnaireList = patient.getQuestionnaires();

        System.out.println("BEFORE:" + questionnaire + " " + patientQuestionnaireList);

        switch (questionnaire) {
            case "CLINICAL-SCCAI":
                if (questionnaireClinicalSCCAIRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "EORTC-QLQ":
                if (questionnaireEORTCQLQRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "EQ5D5L":
                if (questionnaireEQ5D5LRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()){
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "HADS":
                if (questionnaireHADSRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "HBI":
                if (questionnaireHBIRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()){
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "IBD-DISK":
                if (questionnaireIbdDiskRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()){
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "IPAQ-SF":
                if (questionnaireIPAQSFRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "MIAH-CD":
                if (questionnaireMIAHCDRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "MIAH-UC":
                if (questionnaireMIAHUCRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "MMAS8":
                if (questionnaireMMAS8Repository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "PATIENT-SCCAI":
                if (questionnairePatientSCCAIRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "PHQ9":
                if (questionnairePHQ9Repository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "PRISM":
                if (questionnairePRISMRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "PSQI":
                if (questionnairePSQIRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()) {
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "SIBDQ":
                if (questionnaireSIBDQRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()){
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "TSQM":
                if (questionnaireTSQMRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()){
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;

            case "WPAI-UC":
                if (questionnaireWPAIUCRepository.findAllByTypeAndPatientSSN(questionnaire, patientSSN).isEmpty()){
                    patientQuestionnaireList.remove(questionnaire);
                }
                break;
        }

        System.out.println("AFTER:" + patientQuestionnaireList);

        patient.setQuestionnaires(patientQuestionnaireList);

        patients.add(patientForDoctor);
        doctor.setPatients(patients);
        //userRepository.deleteByEmail(doctor.getEmail());
        //userRepository.deleteByEmail(patient.getEmail());

        Optional<ArrayList<Job>> optionalJobs = jobRepository.findAllByIdContaining(patientSSN + "/" + questionnaire);

        if (optionalJobs.isPresent()) {
            ArrayList<Job> jobsForPatient = optionalJobs.get();
            jobsForPatient.forEach(j -> {
                schedulerService.killJobById(j.getId());
            });
        }

        userRepository.save(doctor);
        userRepository.save(patient);

        String body = "Gentile " + patient.getName() + " " + patient.getSurname() + "\n" +
                "è stata rimossa la tipologia di questionario " + questionnaire + " per il tuo account.\n";

        String subject = "IBD Tool - Rimossa tipologia di questionario " + questionnaire;

        this.notificationService.sendNotificationToUser(patient, subject, body);
    }

    /**
     * Get All pendings questionnaires
     * @param patientSSN
     * @return
     * @throws UserNotFoundException
     * @throws QuestionnaireNotFoundException
     */
    public ArrayList<Pending> getAllPending(String patientSSN) throws UserNotFoundException, QuestionnaireNotFoundException {
        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);

        if (optionalPatient.isEmpty())
            throw new UserNotFoundException();

        Optional<ArrayList<Pending>> optionalPendingArrayList = pendingRepository.findByPatientSSN(patientSSN);

        if (optionalPendingArrayList.isEmpty())
            throw new QuestionnaireNotFoundException();

        return optionalPendingArrayList.get();
    }

    /**
     * Create and send a new questionnaire to the patient
     * @param type
     * @param doctorID
     * @param patientSSN
     * @return
     * @throws UserNotFoundException
     * @throws QuestionnaireAlreadyCreatedException
     * @throws QuestionnaireAlreadyCompiledException
     */
    public boolean createAndSend(String type, String doctorID, String patientSSN) throws QuestionnaireAlreadyCreatedException, QuestionnaireAlreadyCompiledException, IOException {

        System.out.println("CREATE AND SEND at: " + System.currentTimeMillis());
        Optional<UserEntity> optDoctor = userRepository.findByDoctorID(doctorID);
        Optional<UserEntity> optPatient = userRepository.findBySSN(patientSSN);

        if (optDoctor.isEmpty())
            throw new UserNotFoundException();

        if (optPatient.isEmpty())
            throw new UserNotFoundException();

        UserEntity patient = optPatient.get();
        UserEntity doctor = optDoctor.get();

        LocalDateTime now = LocalDateTime.now(ZoneOffset.UTC);
        String date = now.format(DateTimeFormatter.ofPattern("dd/MM/yyyy"));

        String id = date + "-" + type + "-" + doctorID + "-" + patientSSN;

        Optional<Pending> optionalPending = pendingRepository.findById(id);

        if (optionalPending.isPresent())
            throw new QuestionnaireAlreadyCreatedException();

        switch (type) {
            case "CLINICAL-SCCAI":
                if (questionnaireClinicalSCCAIRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "EORTC-QLQ":
                if (questionnaireEORTCQLQRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "EQ5D5L":
                if (questionnaireEQ5D5LRepository.findById(id).isPresent()){
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "HADS":
                if (questionnaireHADSRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "HBI":
                if (questionnaireHBIRepository.findById(id).isPresent()){
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "IBD-DISK":
                if (questionnaireIbdDiskRepository.findById(id).isPresent()){
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "IPAQ-SF":
                if (questionnaireIPAQSFRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "MIAH-CD":
                if (questionnaireMIAHCDRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "MIAH-UC":
                if (questionnaireMIAHUCRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "MMAS8":
                if (questionnaireMMAS8Repository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "PATIENT-SCCAI":
                if (questionnairePatientSCCAIRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "PHQ9":
                if (questionnairePHQ9Repository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "PRISM":
                if (questionnairePRISMRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "PSQI":
                if (questionnairePSQIRepository.findById(id).isPresent()) {
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "SIBDQ":
                if (questionnaireSIBDQRepository.findById(id).isPresent()){
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "TSQM":
                if (questionnaireTSQMRepository.findById(id).isPresent()){
                    throw new QuestionnaireAlreadyCompiledException();
                }
            case "WPAI-UC":
                if (questionnaireWPAIUCRepository.findById(id).isPresent()){
                    throw new QuestionnaireAlreadyCompiledException();
                }

        }

        String uuid = UUID.randomUUID().toString();

        Pending pending = Pending.builder()
                .id(id)
                .uuid(uuid)
                .type(type)
                .doctorID(doctorID)
                .patientSSN(patientSSN)
                .date(now)
                .build();

        pendingRepository.save(pending);

        String email = patient.getEmail();
        String subject = "IBD Tool - Nuovo questionario " + type + " da compilare";
        String message = "Gentile " + patient.getName() + " " + patient.getSurname() + "\n" +
                "è disponibile un nuovo questionario " + type + " da compilare sulla tua pagina personale.\n" +
                "Ti preghiamo di compilarlo al più presto.\n" +
                "Puoi compilarlo direttamente cliccando qui: " +
                link + "/questionnaire-compile/" + type.toLowerCase() + "/" + uuid + "/" + doctorID + "/" + patientSSN + "\n" +
                "Cordiali saluti,\n" +
                "IBD Tool - Ospedale Mauriziano";


        if (type.toLowerCase().equals("clinical-sccai")) {
            email = doctor.getEmail();
            message = "Gentile " + doctor.getName() + " " + doctor.getSurname() + "\n" +
                    "è disponibile un nuovo questionario " + type + " da compilare relativo al paziente " + patient.getName() + " " + patient.getSurname() + " (codice fiscale: " + patientSSN + ") \n" +
                    "\nPuoi compilarlo direttamente cliccando qui: " +
                    link + "/questionnaire-compile/" + type.toLowerCase() + "/" + uuid + "/" + doctorID + "/" + patientSSN + "\n" +
                    "Cordiali saluti,\n" +
                    "IBD Tool - Ospedale Mauriziano";

            String body = "Gentile " + doctor.getName() + " " + doctor.getSurname() + "\n" +
                    "è disponibile un nuovo questionario " + type + " da compilare relativo al paziente " + patient.getName() + " " + patient.getSurname() + " (codice fiscale: " + patientSSN + ") \n" +
                    "Clicca qui: " +
                    link + "/questionnaire-compile/" + type.toLowerCase() + "/" + uuid + "/" + doctorID + "/" + patientSSN + " \n";

            this.notificationService.sendNotificationToUser(doctor, subject, body);
        }

        else {
            String body = "Gentile " + patient.getName() + " " + patient.getSurname() + "\n" +
                    "è disponibile un nuovo questionario " + type + " da compilare.\n" +
                    "Clicca qui: " +
                    link + "/questionnaire-compile/" + type.toLowerCase() + "/" + uuid + "/" + doctorID + "/" + patientSSN + "\n";

            this.notificationService.sendNotificationToUser(patient, subject, body);
        }

        mailService.sendMail(email, subject, message);

        return true;
    }

    /**
     * get Updates from patient
     * @param patientSSN
     * @return
     */
    public HashSet<String> getUpdatedQuestionnaires(String patientSSN) throws UserNotFoundException, RequestNotAuthorizedException {

        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);
        if (optionalPatient.isEmpty())
            throw new UserNotFoundException();

        UserEntity patient = optionalPatient.get();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.getEmail()))
            throw new RequestNotAuthorizedException();

        return patient.getQuestionnaires();
    }

    /**
     * Read questionnaire
     * @param date
     * @param type
     * @param doctorID
     * @param patientSSN
     * @throws UserNotFoundException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     */
    public void readQuestionnaire(String date, String type, String doctorID, String patientSSN) throws UserNotFoundException, RequestNotAuthorizedException, QuestionnaireNotFoundException {

        Optional<UserEntity> optionalDoctor = userRepository.findByDoctorID(doctorID);

        if (optionalDoctor.isEmpty())
            throw new UserNotFoundException();

        UserEntity doctor = optionalDoctor.get();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(doctor.getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<?> optionalQuestionnaire;

        switch (type){
            case "CLINICAL-SCCAI":
                optionalQuestionnaire = questionnaireClinicalSCCAIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireClinicalSCCAI questionnaire = (QuestionnaireClinicalSCCAI) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireClinicalSCCAIRepository.save(questionnaire);
                }
                break;
            case "EORTC-QLQ":
                optionalQuestionnaire = questionnaireEORTCQLQRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireEORTC_QLQ questionnaire = (QuestionnaireEORTC_QLQ) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireEORTCQLQRepository.save(questionnaire);
                }
                break;
            case "EQ5D5L":
                optionalQuestionnaire = questionnaireEQ5D5LRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireEQ5D5L questionnaire = (QuestionnaireEQ5D5L) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireEQ5D5LRepository.save(questionnaire);
                }
                break;
            case "HADS":
                optionalQuestionnaire = questionnaireHADSRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireHADS questionnaire = (QuestionnaireHADS) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireHADSRepository.save(questionnaire);
                }
                break;
            case "HBI":
                optionalQuestionnaire = questionnaireHBIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireHBI questionnaire = (QuestionnaireHBI) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireHBIRepository.save(questionnaire);
                }
                break;
            case "IBD-DISK":
                optionalQuestionnaire = questionnaireIbdDiskRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireIBDDisk questionnaire = (QuestionnaireIBDDisk) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireIbdDiskRepository.save(questionnaire);
                }
                break;
            case "IPAQ-SF":
                optionalQuestionnaire = questionnaireIPAQSFRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireIPAQSF questionnaire = (QuestionnaireIPAQSF) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireIPAQSFRepository.save(questionnaire);
                }
                break;

            case "MIAH-CD":
                optionalQuestionnaire = questionnaireMIAHCDRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMIAHCD questionnaire = (QuestionnaireMIAHCD) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireMIAHCDRepository.save(questionnaire);
                }
                break;

            case "MIAH-UC":
                optionalQuestionnaire = questionnaireMIAHUCRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMIAHUC questionnaire = (QuestionnaireMIAHUC) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireMIAHUCRepository.save(questionnaire);
                }
                break;

            case "MMAS8":
                optionalQuestionnaire = questionnaireMMAS8Repository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMMAS8 questionnaire = (QuestionnaireMMAS8) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireMMAS8Repository.save(questionnaire);
                }
                break;

            case "PATIENT-SCCAI":
                optionalQuestionnaire = questionnairePatientSCCAIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePatientSCCAI questionnaire = (QuestionnairePatientSCCAI) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnairePatientSCCAIRepository.save(questionnaire);
                }
                break;

            case "PHQ9":
                optionalQuestionnaire = questionnairePHQ9Repository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePHQ9 questionnaire = (QuestionnairePHQ9) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnairePHQ9Repository.save(questionnaire);
                }
                break;
            case "PRISM":
                optionalQuestionnaire = questionnairePRISMRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePRISM questionnaire = (QuestionnairePRISM) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnairePRISMRepository.save(questionnaire);
                }
                break;
            case "PSQI":
                optionalQuestionnaire = questionnairePSQIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePSQI questionnaire = (QuestionnairePSQI) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnairePSQIRepository.save(questionnaire);
                }
                break;

            case "SIBDQ":
                optionalQuestionnaire = questionnaireSIBDQRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireSIBDQ questionnaire = (QuestionnaireSIBDQ) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireSIBDQRepository.save(questionnaire);
                }
                break;
            case "TSQM":
                optionalQuestionnaire = questionnaireTSQMRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireTSQM questionnaire = (QuestionnaireTSQM) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireTSQMRepository.save(questionnaire);
                }
                break;
            case "WPAI-UC":
                optionalQuestionnaire = questionnaireWPAIUCRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireWPAIUC questionnaire = (QuestionnaireWPAIUC) optionalQuestionnaire.get();
                    questionnaire.setRead(true);
                    questionnaireWPAIUCRepository.save(questionnaire);
                }
                break;

            default:
                break;
        }

    }

    /**
     * Warn doctor
     * @param date
     * @param type
     * @param doctorID
     * @param patientSSN
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     */
    public void warnDoctor(String date, String type, String doctorID, String patientSSN) throws IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException {
        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);

        if (optionalPatient.isEmpty())
            throw new UserNotFoundException();

        UserEntity patient = optionalPatient.get();

        Optional<UserEntity> optionalDoctor = userRepository.findByDoctorID(doctorID);

        if (optionalDoctor.isEmpty())
            throw new UserNotFoundException();

        UserEntity doctor = optionalDoctor.get();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(patient.getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<?> optionalQuestionnaire;

        switch (type){
            case "CLINICAL-SCCAI":
                optionalQuestionnaire = questionnaireClinicalSCCAIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireClinicalSCCAI questionnaire = (QuestionnaireClinicalSCCAI) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireClinicalSCCAIRepository.save(questionnaire);
                }
                break;
            case "EORTC-QLQ":
                optionalQuestionnaire = questionnaireEORTCQLQRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireEORTC_QLQ questionnaire = (QuestionnaireEORTC_QLQ) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireEORTCQLQRepository.save(questionnaire);
                }
                break;
            case "EQ5D5L":
                optionalQuestionnaire = questionnaireEQ5D5LRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireEQ5D5L questionnaire = (QuestionnaireEQ5D5L) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireEQ5D5LRepository.save(questionnaire);
                }
                break;
            case "HADS":
                optionalQuestionnaire = questionnaireHADSRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireHADS questionnaire = (QuestionnaireHADS) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireHADSRepository.save(questionnaire);
                }
                break;
            case "HBI":
                optionalQuestionnaire = questionnaireHBIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireHBI questionnaire = (QuestionnaireHBI) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireHBIRepository.save(questionnaire);
                }
                break;
            case "IBD-DISK":
                optionalQuestionnaire = questionnaireIbdDiskRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireIBDDisk questionnaire = (QuestionnaireIBDDisk) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireIbdDiskRepository.save(questionnaire);
                }
                break;
            case "IPAQ-SF":
                optionalQuestionnaire = questionnaireIPAQSFRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireIPAQSF questionnaire = (QuestionnaireIPAQSF) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireIPAQSFRepository.save(questionnaire);
                }
                break;
            case "MIAH-CD":
                optionalQuestionnaire = questionnaireMIAHCDRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMIAHCD questionnaire = (QuestionnaireMIAHCD) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireMIAHCDRepository.save(questionnaire);
                }
                break;
            case "MIAH-UC":
                optionalQuestionnaire = questionnaireMIAHUCRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMIAHUC questionnaire = (QuestionnaireMIAHUC) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireMIAHUCRepository.save(questionnaire);
                }
                break;
            case "MMAS8":
                optionalQuestionnaire = questionnaireMMAS8Repository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMMAS8 questionnaire = (QuestionnaireMMAS8) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireMMAS8Repository.save(questionnaire);
                }
                break;
            case "PATIENT-SCCAI":
                optionalQuestionnaire = questionnairePatientSCCAIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePatientSCCAI questionnaire = (QuestionnairePatientSCCAI) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnairePatientSCCAIRepository.save(questionnaire);
                }
                break;
            case "PHQ9":
                optionalQuestionnaire = questionnairePHQ9Repository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePHQ9 questionnaire = (QuestionnairePHQ9) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnairePHQ9Repository.save(questionnaire);
                }
                break;
            case "PRISM":
                optionalQuestionnaire = questionnairePRISMRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePRISM questionnaire = (QuestionnairePRISM) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnairePRISMRepository.save(questionnaire);
                }
                break;
            case "PSQI":
                optionalQuestionnaire = questionnairePSQIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePSQI questionnaire = (QuestionnairePSQI) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnairePSQIRepository.save(questionnaire);
                }
                break;
            case "SIBDQ":
                optionalQuestionnaire = questionnaireSIBDQRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireSIBDQ questionnaire = (QuestionnaireSIBDQ) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireSIBDQRepository.save(questionnaire);
                }
                break;
            case "TSQM":
                optionalQuestionnaire = questionnaireTSQMRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireTSQM questionnaire = (QuestionnaireTSQM) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireTSQMRepository.save(questionnaire);
                }
                break;
            case "WPAI-UC":
                optionalQuestionnaire = questionnaireWPAIUCRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(date, type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireWPAIUC questionnaire = (QuestionnaireWPAIUC) optionalQuestionnaire.get();
                    questionnaire.setWarning(true);
                    questionnaire.setEvaluation(false);
                    questionnaireWPAIUCRepository.save(questionnaire);
                }
                break;

            default:
                break;
        }

        String subject = "IBD Tool - Segnalazione questionario " + type + " del " + date;
        String body = "Gentile " + doctor.getName() + " " + doctor.getSurname() + ",\n" +
                "il paziente " + patient.getName() + " " + patient.getSurname() + " (codice fiscale: " + patient.getSSN() + ") " +
                "ha segnalato la compilazione errata del questionario " + type + " del " + date;

        this.notificationService.sendNotificationToUser(doctor, subject, body);
    }


    /**
     * Evaluate questionnaire
     * @param type
     * @param doctorID
     * @param patientSSN
     * @param evaluationDTO
     * @throws IOException
     * @throws RequestNotAuthorizedException
     * @throws QuestionnaireNotFoundException
     */
    public void evaluateQuestionnaire(String type, String doctorID, String patientSSN, EvaluationDTO evaluationDTO) throws IOException, RequestNotAuthorizedException, QuestionnaireNotFoundException {

        Optional<UserEntity> optionalPatient = userRepository.findBySSN(patientSSN);

        if (optionalPatient.isEmpty())
            throw new UserNotFoundException();

        UserEntity patient = optionalPatient.get();

        Optional<UserEntity> optionalDoctor = userRepository.findByDoctorID(doctorID);

        if (optionalDoctor.isEmpty())
            throw new UserNotFoundException();

        UserEntity doctor = optionalDoctor.get();

        this.authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.getPrincipal().toString().equals(doctor.getEmail()))
            throw new RequestNotAuthorizedException();

        Optional<?> optionalQuestionnaire;

        switch (type){
            case "CLINICAL-SCCAI":
                optionalQuestionnaire = questionnaireClinicalSCCAIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireClinicalSCCAI questionnaire = (QuestionnaireClinicalSCCAI) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireClinicalSCCAIRepository.save(questionnaire);
                }
                break;
            case "EORTC-QLQ":
                optionalQuestionnaire = questionnaireEORTCQLQRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireEORTC_QLQ questionnaire = (QuestionnaireEORTC_QLQ) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireEORTCQLQRepository.save(questionnaire);
                }
                break;
            case "EQ5D5L":
                optionalQuestionnaire = questionnaireEQ5D5LRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireEQ5D5L questionnaire = (QuestionnaireEQ5D5L) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireEQ5D5LRepository.save(questionnaire);
                }
                break;
            case "HADS":
                optionalQuestionnaire = questionnaireHADSRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireHADS questionnaire = (QuestionnaireHADS) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireHADSRepository.save(questionnaire);
                }
                break;
            case "HBI":
                optionalQuestionnaire = questionnaireHBIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireHBI questionnaire = (QuestionnaireHBI) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireHBIRepository.save(questionnaire);
                }
                break;
            case "IBD-DISK":
                optionalQuestionnaire = questionnaireIbdDiskRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireIBDDisk questionnaire = (QuestionnaireIBDDisk) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireIbdDiskRepository.save(questionnaire);
                }
                break;
            case "IPAQ-SF":
                optionalQuestionnaire = questionnaireIPAQSFRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireIPAQSF questionnaire = (QuestionnaireIPAQSF) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireIPAQSFRepository.save(questionnaire);
                }
                break;
            case "MIAH-CD":
                optionalQuestionnaire = questionnaireMIAHCDRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMIAHCD questionnaire = (QuestionnaireMIAHCD) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireMIAHCDRepository.save(questionnaire);
                }
                break;
            case "MIAH-UC":
                optionalQuestionnaire = questionnaireMIAHUCRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMIAHUC questionnaire = (QuestionnaireMIAHUC) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireMIAHUCRepository.save(questionnaire);
                }
                break;
            case "MMAS8":
                optionalQuestionnaire = questionnaireMMAS8Repository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireMMAS8 questionnaire = (QuestionnaireMMAS8) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireMMAS8Repository.save(questionnaire);
                }
                break;
            case "PATIENT-SCCAI":
                optionalQuestionnaire = questionnairePatientSCCAIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePatientSCCAI questionnaire = (QuestionnairePatientSCCAI) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnairePatientSCCAIRepository.save(questionnaire);
                }
                break;
            case "PHQ9":
                optionalQuestionnaire = questionnairePHQ9Repository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePHQ9 questionnaire = (QuestionnairePHQ9) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnairePHQ9Repository.save(questionnaire);
                }
                break;
            case "PRISM":
                optionalQuestionnaire = questionnairePRISMRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePRISM questionnaire = (QuestionnairePRISM) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnairePRISMRepository.save(questionnaire);
                }
                break;
            case "PSQI":
                optionalQuestionnaire = questionnairePSQIRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnairePSQI questionnaire = (QuestionnairePSQI) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnairePSQIRepository.save(questionnaire);
                }
                break;
            case "SIBDQ":
                optionalQuestionnaire = questionnaireSIBDQRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireSIBDQ questionnaire = (QuestionnaireSIBDQ) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireSIBDQRepository.save(questionnaire);
                }
                break;
            case "TSQM":
                optionalQuestionnaire = questionnaireTSQMRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireTSQM questionnaire = (QuestionnaireTSQM) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireTSQMRepository.save(questionnaire);
                }
                break;
            case "WPAI-UC":
                optionalQuestionnaire = questionnaireWPAIUCRepository.findByIdContainingAndTypeAndDoctorIDAndPatientSSN(evaluationDTO.getDate(), type, doctorID, patientSSN);
                if (optionalQuestionnaire.isEmpty())
                    throw new QuestionnaireNotFoundException();
                else {
                    QuestionnaireWPAIUC questionnaire = (QuestionnaireWPAIUC) optionalQuestionnaire.get();
                    if (evaluationDTO.isEvaluation()) {
                        questionnaire.setEvaluation(true);
                        questionnaire.setWarning(false);
                    }
                    else {
                        questionnaire.setEvaluation(false);
                        questionnaire.setWarning(true);
                    }
                    questionnaireWPAIUCRepository.save(questionnaire);
                }
                break;

            default:
                break;
        }

        String subject = "IBD Tool - Valutazione questionario " + type + " del " + evaluationDTO.getDate();
        String body = "Gentile " + patient.getName() + " " + patient.getSurname() + ",\n" +
                "il tuo medico ha visionato il questionario " + type + " del " + evaluationDTO.getDate();

        if (evaluationDTO.isEvaluation()) {
            body += " ed è stato valutato correttamente.";
        }

        else {
            body += " e lo ha segnalato";
        }


        if (!type.toUpperCase().equals("CLINICAL-SCCAI")) {
            this.notificationService.sendNotificationToUser(patient, subject, body);
        }
    }


    // EMERGENZA PER INVIO QUESTIONARI A TUTTI IL 14/08
    /*public void globalSending() throws QuestionnaireAlreadyCreatedException, IOException, QuestionnaireAlreadyCompiledException {
        Optional<ArrayList<UserEntity>> optionalPatients = userRepository.findAllByDoctorExists(true);
        assert(optionalPatients.isPresent());
        ArrayList<UserEntity> patients = optionalPatients.get();

        for (UserEntity p : patients) {
            if (p.getBodyStat().getPathology().equals("Malattia di Crohn")) {
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "HBI").isEmpty()
                    && questionnaireHBIRepository.findByIdContaining("14/08/2020-HBI-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("HBI", p.getDoctor().getDoctorID(), p.getSSN());
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "MIAH-CD").isEmpty()
                        && questionnaireMIAHCDRepository.findByIdContaining("14/08/2020-MIAH-CD-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("MIAH-CD", p.getDoctor().getDoctorID(), p.getSSN());
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "IBD-DISK").isEmpty()
                        && questionnaireIbdDiskRepository.findByIdContaining("14/08/2020-IBD-DISK-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("IBD-DISK", p.getDoctor().getDoctorID(), p.getSSN());
            } else if (p.getBodyStat().getPathology().equals("Colite ulcerosa")) {
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "PATIENT-SCCAI").isEmpty()
                        && questionnairePatientSCCAIRepository.findByIdContaining("14/08/2020-PATIENT-SCCAI-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("PATIENT-SCCAI", p.getDoctor().getDoctorID(), p.getSSN());
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "MIAH-UC").isEmpty()
                        && questionnaireMIAHUCRepository.findByIdContaining("14/08/2020-MIAH-UC-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("MIAH-UC", p.getDoctor().getDoctorID(), p.getSSN());
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "IBD-DISK").isEmpty()
                        && questionnaireIbdDiskRepository.findByIdContaining("14/08/2020-IBD-DISK-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("IBD-DISK", p.getDoctor().getDoctorID(), p.getSSN());
            } else {
                if (pendingRepository.findByPatientSSNAndType(p.getSSN(), "IBD-DISK").isEmpty()
                        && questionnaireIbdDiskRepository.findByIdContaining("14/08/2020-IBD-DISK-" + p.getDoctor().getDoctorID() + "-" + p.getSSN()).isEmpty())
                    this.createAndSend("IBD-DISK", p.getDoctor().getDoctorID(), p.getSSN());
            }
        }
    }*/
}

