import { Component, OnInit } from '@angular/core';
import {Observable} from 'rxjs';
import {Store} from '@ngrx/store';
import {AppState, selectAuthState, selectDoctorState} from '../../../store/app.states';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {RegistrationService} from '../../../services/registration.service';
import {PatientRegistration} from '../../../model/user/PatientRegistration';
import {BodyStat} from '../../../model/user/BodyStat';
import {MatDatepickerInputEvent} from '@angular/material/datepicker';
import {DateAdapter, MatOptionSelectionChange} from '@angular/material/core';
import {Router} from '@angular/router';
import {Patient} from '../../../model/user/Patient';
import {GetPatients} from '../../../store/actions/doctor.actions';

@Component({
  selector: 'app-patient-registration',
  templateUrl: './patient-registration.component.html',
  styleUrls: ['./patient-registration.component.css'],
})

export class PatientRegistrationComponent implements OnInit {

  getAuthState: Observable<any>;
  getDoctorState: Observable<any>;
  isAuthenticated: boolean;
  user = null;
  errorMessage = null;
  patients: Patient[];
  title = 'Registrazione paziente';
  registrationForm: FormGroup;
  notFocused = false;
  birthDate: string;
  patientRegistrationBody: PatientRegistration;
  submitted: boolean;
  show = false;
  destination = 'patients-control-panel';
  pathologies = ['Malattia di Crohn', 'Colite ulcerosa', 'Altro'];
  otherPathology = false;
  choices = ['Random', 'Telemedicina', 'Cure tradizionali'];

  constructor(private store: Store<AppState>,
              private registrationService: RegistrationService,
              private dateAdapter: DateAdapter<Date>,
              private router: Router) {
    this.getAuthState = this.store.select(selectAuthState);
    this.getDoctorState = this.store.select(selectDoctorState);
    this.dateAdapter.setLocale('it');
  }

  ngOnInit(): void {
    this.getAuthState.subscribe((state) => {
      this.isAuthenticated = state.isAuthenticated;
      this.user = JSON.parse(localStorage.getItem('user'));
      this.errorMessage = state.errorMessage;
    });

    this.registrationForm = new FormGroup({
      email: new FormControl('', [Validators.required, Validators.email]),
      name: new FormControl('', [Validators.required]),
      surname: new FormControl('', [Validators.required]),
      ssn: new FormControl('', [Validators.required]),
      // birthPlace: new FormControl('', [Validators.required]),
      // province: new FormControl('', [Validators.required]),
      // cap: new FormControl('', [Validators.required]),
      // phoneNumber: new FormControl('', [Validators.required]),
      // weight: new FormControl('', [Validators.required, Validators.min(0)]),
      // height: new FormControl('', [Validators.required, Validators.min(0)]),
      pathology: new FormControl('', [Validators.required]),
      // category: new FormControl('', [Validators.required]),
    });

  }

  onSubmit(registration) {
    this.submitted = true;
    this.show = true;
    console.log(registration);
    const bodyStat = new BodyStat(/*registration.weight +*/ ' kg', /*registration.height +*/ ' cm', registration.pathology);
    // tslint:disable-next-line:max-line-length
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    this.patientRegistrationBody =
      // tslint:disable-next-line:max-line-length
      new PatientRegistration(
        registration.email,
        registration.name,
        registration.surname,
        registration.ssn,
        this.birthDate,
        // registration.birthPlace + ' (' + registration.province + ') - ' + registration.cap,
        // registration.phoneNumber,
        doctorID,
        bodyStat);

    console.log('CATEGORY:', registration.category);
    console.log(this.patientRegistrationBody);

    // ONLY RANDOM

    this.registrationService.registerPatient(this.patientRegistrationBody, 'RANDOM').subscribe(d => {
        if (d.status === 200) {
          // tslint:disable-next-line:only-arrow-functions
          this.router.navigateByUrl('/success').then(() => setTimeout(function() {
            location.replace('/patients-control-panel');
          }, 1500));
        }
      },
      error => {
        console.log(error);
        // tslint:disable-next-line:only-arrow-functions
        this.router.navigateByUrl('/error').then(() => setTimeout(function() {
          location.replace('/patients-control-panel');
        }, 2000));
      });
  }

  setDate($event: MatDatepickerInputEvent<any>) {
    this.birthDate =
      $event.target._dateAdapter.getDate($event.target.value).toString(10) + '/' +
      ($event.target._dateAdapter.getMonth($event.target.value) + 1).toString(10) + '/' +
      $event.target._dateAdapter.getYear($event.target.value).toString(10);

    console.log(this.birthDate);
  }

  selectResult($event: MatOptionSelectionChange, value: string) {
    if ($event.isUserInput) {
      /*if (value.localeCompare('Altro') !== 0) { // per specificare altra patologia -> non più richiesto
        this.registrationForm.get('pathology').setValue(value);
        this.otherPathology = false;
      } else {
        this.registrationForm.get('pathology').reset();
        this.otherPathology = true;
      }*/
      this.registrationForm.get('pathology').setValue(value);
      this.otherPathology = false;
      console.log(this.registrationForm.get('pathology').value);
    }
  }

  selectChoice($event: MatOptionSelectionChange, value: string) {
    if ($event.isUserInput) {
      if (value.localeCompare('Random') === 0) {
        this.registrationForm.get('category').setValue('RANDOM');
      } else if (value.localeCompare('Telemedicina') === 0) {
        this.registrationForm.get('category').setValue('TELEMEDICINA');
      } else {
        this.registrationForm.get('category').setValue('STANDARD');
      }
      console.log(this.registrationForm.get('category').value);
    }
  }
}
