import {Component, OnInit, ViewChild} from '@angular/core';
import {Observable} from 'rxjs';
import {Patient} from '../../../model/user/Patient';
import {MatPaginator} from '@angular/material/paginator';
import {MatSort} from '@angular/material/sort';
import {Store} from '@ngrx/store';
import {AppState, selectAuthState, selectDoctorState} from '../../../store/app.states';
import {Router} from '@angular/router';
import {DoctorService} from '../../../services/doctor.service';
import {MatDialog} from '@angular/material/dialog';
import {GetPatients} from '../../../store/actions/doctor.actions';
import {MatTableDataSource} from '@angular/material/table';
import {ConfirmDialogComponent} from '../../user-actions/confirm-dialog/confirm-dialog.component';
import {ErrorDialogComponent} from '../../user-actions/error-dialog/error-dialog.component';

@Component({
  selector: 'app-patients-global-control-panel',
  templateUrl: './patients-global-control-panel.component.html',
  styleUrls: ['./patients-global-control-panel.component.css']
})
export class PatientsGlobalControlPanelComponent implements OnInit {

  getAuthState: Observable<any>;
  getDoctorState: Observable<any>;
  isAuthenticated: boolean;
  user = null;
  errorMessage = null;
  patients: Patient[];
  dataSource;
  title = 'Pazienti registrati';
  displayedColumns = ['surname', 'name', 'ssn', 'doctor', 'category', 'card', 'questionnaires', 'email'];
  patientSSN: string;

  @ViewChild(MatPaginator, {static: true}) paginator: MatPaginator;
  @ViewChild(MatSort, {static: true}) sort: MatSort;

  constructor(private store: Store<AppState>,
              private router: Router,
              private doctorService: DoctorService,
              public dialog: MatDialog) {
    this.getAuthState = this.store.select(selectAuthState);
    this.getDoctorState = this.store.select(selectDoctorState);
    document.addEventListener('notification', () => {
      this.doctorService.getGlobalPatients().subscribe((patients) => {
        localStorage.setItem('patients', JSON.stringify(patients));
      });
      setTimeout(() => {
        this.patients = JSON.parse(localStorage.getItem('patients'));
        this.dataSource.data = this.patients ? this.patients : [];
      }, 500);
    });
  }

  ngOnInit(): void {
    this.paginator._intl.itemsPerPageLabel = 'Pazienti per pagina';
    this.getAuthState.subscribe((state) => {
      this.isAuthenticated = state.isAuthenticated;
      this.user = JSON.parse(localStorage.getItem('user'));
      this.errorMessage = state.errorMessage;
    });


    this.doctorService.getGlobalPatients().subscribe((patients) => {
      localStorage.setItem('patients', JSON.stringify(patients));
      this.patients = patients;
      console.log(this.patients);
      if (this.patients !== null && this.patients !== undefined) {
        this.patients.sort((p1, p2) => p1.surname < p2.surname ? -1 : 0);
      }

      this.dataSource = new MatTableDataSource<Patient>(this.patients ? this.patients : []);
      this.dataSource.paginator = this.paginator;
      this.localStorageClear();
    });
  }

  localStorageClear() {

    const user = localStorage.getItem('user');
    const jwt = localStorage.getItem('jwt');
    const expirationTime = localStorage.getItem('expirationTime');
    const patients = localStorage.getItem('patients');
    const firebaseToken = localStorage.getItem('firebase-token');
    const doctorID = localStorage.getItem('doctorID');

    localStorage.clear();

    localStorage.setItem('user', user);
    localStorage.setItem('jwt', jwt);
    localStorage.setItem('expirationTime', expirationTime);
    localStorage.setItem('patients', patients);
    localStorage.setItem('firebase-token', firebaseToken);
    localStorage.setItem('doctorID', doctorID);
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();

    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  goToQuestionnaires(patientSSN: string) {
    this.router.navigateByUrl('/patient-questionnaire-panel/' + patientSSN + '/global');
  }

  /*openDeleteDialog(patient) {
    const message = 'Se confermata, questa operazione non potrà più essere annullata.';
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '500px',
      height: '265px',
      data: message
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result === 'SI') {
        this.doctorService.removePatient(patient).subscribe(d => {
            if (d.status === 200) {
              this.patients.forEach( (p, index) => {
                if (p.ssn.localeCompare(patient.ssn) === 0) {
                  this.patients.splice(index, 1);
                }
              });
              this.dataSource.data = this.patients;
              this.dataSource.sort = this.sort;
              this.doctorService.getGlobalPatients().subscribe((patients) => {
                localStorage.setItem('patients', JSON.stringify(patients));
              });
            }
          },
          error => {
            console.log(error);
            this.openErrorDialog('Qualcosa è andato storto. Le modifiche non verranno salvate.');
          });
      }
    });
  }*/

  openErrorDialog(message: string): void {
    const dialogRef = this.dialog.open(ErrorDialogComponent, {
      width: '500px',
      height: '270px',
      data: message
    });

    dialogRef.afterClosed().subscribe(result => {
      console.log('Finestra chiusa');
    });
  }

  openPanel(patient) {
    this.router.navigateByUrl('/patient-panel/' + patient.ssn + '/global');
  }

}
