import { Component, OnInit } from '@angular/core';
import {AppState, selectAuthState, selectDoctorState, selectPatientState} from '../../../store/app.states';
import {Store} from '@ngrx/store';
import {Observable} from 'rxjs';
import {GetPatients} from '../../../store/actions/doctor.actions';
import {GetPendings} from '../../../store/actions/patient.actions';
import {AuthService} from '../../../services/auth.service';
import {ErrorDialogComponent} from '../../user-actions/error-dialog/error-dialog.component';
import {MatDialog} from '@angular/material/dialog';
import {GdprDialogComponent} from '../../user-actions/gdpr-dialog/gdpr-dialog.component';
import {LogOut} from '../../../store/actions/auth.actions';
import {coerceBooleanProperty} from '@angular/cdk/coercion';

@Component({
  selector: 'app-landing',
  templateUrl: './landing.component.html',
  styleUrls: ['./landing.component.css']
})
export class LandingComponent implements OnInit {

  getAuthState: Observable<any>;
  customState: Observable<any>;
  isAuthenticated: boolean;
  user = null;
  errorMessage = null;
  message;
  title = 'Benvenuto';

  constructor(private store: Store<AppState>,
              private dialog: MatDialog,
              private authService: AuthService) {
    this.getAuthState = this.store.select(selectAuthState);
  }

  ngOnInit(): void {
    this.getAuthState.subscribe((state) => {
      this.isAuthenticated = state.isAuthenticated;
      this.user = JSON.parse(localStorage.getItem('user'));
      this.errorMessage = state.errorMessage;
    });

    if (this.user.role === 'DOTTORE') {
      this.customState = this.store.select(selectDoctorState);
    }

    if (this.user.role === 'PAZIENTE') {
      let enabled = false;
      this.authService.isPatientEnabled(this.user).subscribe(res => {
          if (res.status === 200) {
            localStorage.setItem('enabled', JSON.stringify(res.body));
            enabled = res.body;
            console.log('ENABLED?', enabled);
            if (!enabled) {
              setTimeout(() => {
                this.openGDPRDialog();
              }, 1000);
            } else {
              localStorage.removeItem('enabled');
            }
          }
        },
        error => {
          console.error(error, 'Failed request');
        }
      );

      this.customState = this.store.select(selectPatientState);
      this.store.dispatch(new GetPendings());
    }

  }

  openGDPRDialog(): void {
    const dialogRef = this.dialog.open(GdprDialogComponent, {
      width: '1000px',
      height: '700px',
      data: this.user.role,
      disableClose: true
    });

    dialogRef.afterClosed().subscribe(result => {
      console.log(result);
      let response: string;
      if (result) {
        response = 'true';
      } else {
        response = 'false';
      }
      this.authService.enablePatient(this.user, response);
      if (!result) {
        this.store.dispatch(new LogOut());
      }
    });
  }

}
