import {Component, Input, OnDestroy, OnInit} from '@angular/core';
import {User} from '../../../model/user/User';
import {Store} from '@ngrx/store';
import {AppState} from '../../../store/app.states';
import {LogOut} from '../../../store/actions/auth.actions';
import {fromEvent, Observable, Subscription} from 'rxjs';

@Component({
  selector: 'app-toolbar',
  templateUrl: './toolbar.component.html',
  styleUrls: ['./toolbar.component.css']
})
export class ToolbarComponent implements OnInit, OnDestroy {

  @Input() user: User;
  @Input() back: boolean;
  @Input() title: string;
  @Input() destination: string;
  isMobile: boolean;

  notifications: number;

  resizeObservable$: Observable<Event>;
  resizeSubscription$: Subscription;

  constructor(private store: Store<AppState>) {

    this.resizeObservable$ = fromEvent(window, 'resize');
    this.resizeSubscription$ = this.resizeObservable$.subscribe(evt => {
      if (window.innerWidth < 600) {
        this.isMobile = true;
      } else {
        this.isMobile = false;
      }
    });

    if (window.innerWidth < 600) {
      this.isMobile = true;
    } else {
      this.isMobile = false;
    }

    document.addEventListener('notification', () => {
      setTimeout(() => {
        this.notifications = 0;
        this.user = JSON.parse(localStorage.getItem('user'));
        this.user.notifications.forEach(n => {
          if (n.read === false) {
            ++this.notifications;
          }
        });
      }, 500);
    });

    document.addEventListener('update-notification', () => {
      setTimeout(() => {
        this.notifications = 0;
        this.user = JSON.parse(localStorage.getItem('user'));
        this.user.notifications.forEach(n => {
          if (n.read === false) {
            ++this.notifications;
          }
        });
      }, 500);
    });
  }

  ngOnInit(): void {
    if (this.destination === undefined) {
      this.destination = 'landing';
    }
    this.setup();

    setTimeout(() => {
      const user = JSON.parse(localStorage.getItem('user'));
      if (this.user !== null && this.user !== undefined) {
        if (user.notifications !== this.user.notifications) {
          this.user = user;
          this.setup();
        }
      }
    }, 1000);
  }

  setup() {
    if (this.user !== null && this.user !== undefined) {
      if (this.user.notifications !== null && this.user.notifications !== undefined) {
        this.notifications = 0;
        this.user.notifications.forEach(n => {
          if (n.read === false) {
            ++this.notifications;
          }
        });
      }
      if (this.notifications === null || this.notifications === undefined) {
        this.notifications = 0;
      }
    } else {
      this.notifications = 0;
    }
  }

  logout(): void {
    this.store.dispatch(new LogOut());
  }

  ngOnDestroy() {
    this.resizeSubscription$.unsubscribe();
  }
}
