import { Component, OnInit } from '@angular/core';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {AuthService} from '../../../services/auth.service';
import {MatDialog} from '@angular/material/dialog';
import {ErrorDialogComponent} from '../../user-actions/error-dialog/error-dialog.component';
import {ActivatedRoute} from '@angular/router';

@Component({
  selector: 'app-password-change-confirm',
  templateUrl: './password-change-confirm.component.html',
  styleUrls: ['./password-change-confirm.component.css']
})
export class PasswordChangeConfirmComponent implements OnInit {

  passwordChangeForm: FormGroup;
  notFocused = false;
  submitted: boolean;
  show = false;
  uuid: string;
  title = 'IBD Tool';

  constructor(private authService: AuthService,
              private route: ActivatedRoute,
              public dialog: MatDialog) {
    this.route.params.subscribe(params => {
      this.uuid = params.uuid;
    });
  }

  ngOnInit(): void {
    this.passwordChangeForm = new FormGroup({
      newPassword: new FormControl('', [Validators.required]),
      confirmNewPassword: new FormControl('', [Validators.required])
    });
  }

  onSubmit(form) {
    this.submitted = true;
    this.show = true;
    console.log(form);
    const f = form;
    /*if (f.oldPassword === f.newPassword) {
      const msg = 'La vecchia e la nuova password coincidono!';
      this.openDialog(msg);
    }*/

    if (f.newPassword !== f.confirmNewPassword) {
      const msg = 'I campi \'Nuova password\' e \'Conferma nuova password\' non coincidono!';
      this.openDialog(msg);
    }

    this.authService.changePassword(f, this.uuid);
  }

  openDialog(message: string): void {
    this.show = true;
    this.submitted = true;
    const dialogRef = this.dialog.open(ErrorDialogComponent, {
      width: '500px',
      height: '270px',
      data: message
    });

    dialogRef.afterClosed().subscribe(result => {
      this.show = false;
      console.log('Finestra chiusa');
      this.submitted = false;
    });
  }

}
