import { Component, OnInit } from '@angular/core';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {Registration} from '../../../model/user/registration';
import {RegistrationService} from '../../../services/registration.service';
import {AppState} from '../../../store/app.states';
import {Store} from '@ngrx/store';
import {MatDatepickerInputEvent} from '@angular/material/datepicker';
import {DateAdapter} from '@angular/material/core';
import {PingService} from '../../../services/ping.service';

@Component({
  selector: 'app-registration',
  templateUrl: './registration.component.html',
  styleUrls: ['./registration.component.css'],
})

export class RegistrationComponent implements OnInit {

  registrationForm: FormGroup;
  notFocused = false;
  registrationBody: Registration;
  submitted: boolean;
  show = false;
  birthDate: string;
  title = 'IBD Tool';

  constructor(private registrationService: RegistrationService,
              private store: Store<AppState>,
              private dateAdapter: DateAdapter<Date>,
              private pingService: PingService) {
    this.dateAdapter.setLocale('it');
  }

  ngOnInit(): void {
    this.pingService.ping();
    this.registrationForm = new FormGroup({
      email: new FormControl('', [Validators.required, Validators.email]),
      name: new FormControl('', [Validators.required]),
      surname: new FormControl('', [Validators.required]),
      ssn: new FormControl('', [Validators.required]),
      phoneNumber: new FormControl('', [Validators.required]),
      // birthPlace: new FormControl('', [Validators.required]),
      // province: new FormControl('', [Validators.required]),
      // cap: new FormControl('', [Validators.required]),
      password: new FormControl('', [Validators.required])
    });
  }

  onSubmit(registration) {
    this.submitted = true;
    this.show = true;
    console.log(registration);
    const val = registration;
    // tslint:disable-next-line:max-line-length
    this.registrationBody = new Registration(
      val.email,
      val.name,
      val.surname,
      val.ssn,
      this.birthDate,
      '',
      val.phoneNumber,
      val.password
    );

    this.registrationService.register(this.registrationBody);
  }

  setDate($event: MatDatepickerInputEvent<any>) {
    this.birthDate =
      $event.target._dateAdapter.getDate($event.target.value).toString(10) + '/' +
      ($event.target._dateAdapter.getMonth($event.target.value) + 1).toString(10) + '/' +
      $event.target._dateAdapter.getYear($event.target.value).toString(10);

    console.log(this.birthDate);
  }
}
