import { Injectable } from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {Router} from '@angular/router';
import {BackendApi, localhost} from '../config/backend-api';
import {Patient} from '../model/user/Patient';
import {
  QuestionnaireClinicalSCCAIResult,
  QuestionnaireEORTCResult,
  QuestionnaireEQ5D5LResult,
  QuestionnaireHADSResult,
  QuestionnaireHBIResult,
  QuestionnaireIBDDiskResult,
  QuestionnaireIPAQSFResult,
  QuestionnaireMIAHCDResult,
  QuestionnaireMIAHUCResult,
  QuestionnaireMMAS8Result,
  QuestionnairePatientSCCAIResult, QuestionnairePHQ9Result, QuestionnairePRISMResult, QuestionnairePSQIResult,
  QuestionnaireSIBDQResult,
  QuestionnaireTSQMResult,
  QuestionnaireWPAIUCResult
} from '../model/questionnaires/questionnaire-results';
import {AppState} from '../store/app.states';
import {Store} from '@ngrx/store';
import {JobModel} from '../model/job/JobModel';
import {QuestionnaireMIAHUC} from '../model/questionnaires/questionnaire-miah-uc';

@Injectable({
  providedIn: 'root'
})
export class DoctorService {

  constructor(private httpClient: HttpClient,
              private router: Router,
              private store: Store<AppState>) { }

  getPatients() {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    console.log('url:' + localhost + BackendApi.GET_PATIENTS + doctorID);
    return this.httpClient.get<Patient[]>(localhost + BackendApi.GET_PATIENTS + doctorID);
  }

  getGlobalPatients() {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    console.log('url:' + localhost + BackendApi.GET_GLOBAL_PATIENTS + doctorID);
    return this.httpClient.get<Patient[]>(localhost + BackendApi.GET_GLOBAL_PATIENTS + doctorID);
  }

  // TODO: MODIFY AFTER A NEW QUESTIONNAIRE MUST BE ADDED

  getQuestionnairesClinicalSCCAI(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireClinicalSCCAIResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesEORTC(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireEORTCResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }
  getQuestionnairesEQ5D5L(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireEQ5D5LResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesHADS(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireHADSResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesHBI(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireHBIResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }
  getQuestionnairesIBDDisk(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireIBDDiskResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesIPAQSF(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireIPAQSFResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesMIAHCD(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireMIAHCDResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesMIAHUC(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireMIAHUCResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesMMAS8(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireMMAS8Result[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesPatientSCCAI(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnairePatientSCCAIResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesPHQ9(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnairePHQ9Result[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesPRISM(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnairePRISMResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesPSQI(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnairePSQIResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  getQuestionnairesSIBDQ(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireSIBDQResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }
  getQuestionnairesTSQM(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireTSQMResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }
  getQuestionnairesWPAIUC(patientSSN: string, type: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<QuestionnaireWPAIUCResult[]>
    (localhost + BackendApi.GET_QUESTIONNAIRES + type + '/' + doctorID + '/' + patientSSN);
  }

  removePatient(patient) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    const patientEmail = patient.email;
    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.REMOVE_PATIENT + doctorID + '/' + patientEmail,
      null, {responseType: 'text', observe: 'response'});
  }

  getJobs(patientSSN) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient.get<JobModel[]>(localhost + BackendApi.GET_JOBS + doctorID + '/' + patientSSN);
  }

  public editPatientClinicalProfile(email, statForm) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.EDIT_PATIENT_CLINICAL_PROFILE + doctorID + '/' + email, statForm, {responseType: 'json', observe: 'response'});
  }

  deleteNote(email: string, n: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.DELETE_NOTE + doctorID + '/' + email, n, {responseType: 'text', observe: 'response'});
  }
}

