import { Injectable } from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {Router} from '@angular/router';
import {BackendApi, localhost} from '../config/backend-api';
import {Pending} from '../model/questionnaires/Pending';
import {Store} from '@ngrx/store';
import {AppState} from '../store/app.states';
import {GetPendings} from '../store/actions/patient.actions';

@Injectable({
  providedIn: 'root'
})
export class QuestionnaireService {

  constructor(private httpClient: HttpClient,
              private store: Store<AppState>,
              private router: Router) { }

  sendForm(qForm, type: string, uuid: string) {
    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.QUESTIONNAIRE_SEND + '/' + type + '/' + uuid, qForm, {responseType: 'text', observe: 'response'})
      .subscribe(d => {
      console.log(d);
      if (d.status === 200) {
        this.store.dispatch(new GetPendings());
        // tslint:disable-next-line:only-arrow-functions
        this.router.navigateByUrl('/success').then(() => setTimeout(function() {
          location.replace('/landing');
        }, 3000));
        // this.router.navigateByUrl('/success');
        }
      },
      error => {
        // tslint:disable-next-line:only-arrow-functions
        this.router.navigateByUrl('/error').then(() => setTimeout(function() {
          location.replace('/landing');
        }, 2000));
        console.log(error);
      });
  }

  setQuestionnaires(patientSSN: string, questionnaires: string[]) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.SET_QUESTIONNAIRES + doctorID + '/' + patientSSN, questionnaires, {responseType: 'text', observe: 'response'});
  }

  removeQuestionnaire(patientSSN: string, questionnaire: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.REMOVE_QUESTIONNAIRE + doctorID + '/' + patientSSN, questionnaire, {responseType: 'text', observe: 'response'});
  }

  createAndSend(questionnaire, patientSSN) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    return this.httpClient
      .post(localhost + BackendApi.CREATE_AND_SEND_QUESTIONNAIRE + questionnaire + '/' + doctorID + '/' + patientSSN, null,
        {responseType: 'text', observe: 'response'});
  }

  getAllPendings() {
    const patientSSN = JSON.parse(localStorage.getItem('user')).ssn;
    return this.httpClient.get<Pending[]>(localhost + BackendApi.GET_ALL_PENDING + patientSSN);
  }

  addJob(type: string, patientSSN: string, timer) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    console.log('Timer: ' + timer);
    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.ADD_JOB + doctorID + '/' + patientSSN + '/' + type, timer, {responseType: 'text', observe: 'response'});
  }

  killJob(type: string, patientSSN: string) {
    const doctorID = JSON.parse(localStorage.getItem('doctorID'));
    // tslint:disable-next-line:max-line-length
    return this.httpClient.post(localhost + BackendApi.KILL_JOB + doctorID + '/' + patientSSN + '/' + type, null, {responseType: 'text', observe: 'response'});
  }
}
