import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { Observable } from 'rxjs';
import 'rxjs/add/observable/of';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/catch';
import {map, switchMap, tap} from 'rxjs/operators';
import {
  DoctorActionTypes,
  GetPatientsSuccess,
  GetPatientsFailure,
  GetQuestionnairesEORTC,
  GetQuestionnairesEORTCSuccess,
  GetQuestionnairesEORTCFailure,
  GetQuestionnairesEQ5D5L,
  GetQuestionnairesEQ5D5LFailure,
  GetQuestionnairesEQ5D5LSuccess,
  GetQuestionnairesHBI,
  GetQuestionnairesHBISuccess,
  GetQuestionnairesHBIFailure,
  GetQuestionnairesIBDDisk,
  GetQuestionnairesIBDDiskSuccess,
  GetQuestionnairesIBDDiskFailure,
  GetQuestionnairesSIBDQ,
  GetQuestionnairesSIBDQSuccess,
  GetQuestionnairesSIBDQFailure,
  GetQuestionnairesTSQMSuccess,
  GetQuestionnairesTSQMFailure,
  GetQuestionnairesWPAIUC,
  GetQuestionnairesWPAIUCSuccess,
  GetQuestionnairesWPAIUCFailure,
  GetQuestionnairesTSQM,
  GetJobs,
  GetJobsSuccess,
  GetQuestionnairesClinicalSCCAI,
  GetQuestionnairesClinicalSCCAISuccess,
  GetQuestionnairesClinicalSCCAIFailure,
  GetQuestionnairesHADS,
  GetQuestionnairesHADSSuccess,
  GetQuestionnairesIPAQSF,
  GetQuestionnairesIPAQSFSuccess,
  GetQuestionnairesMIAHCDSuccess,
  GetQuestionnairesMIAHCDFailure,
  GetQuestionnairesMIAHUC,
  GetQuestionnairesMIAHUCFailure,
  GetQuestionnairesMMAS8Success,
  GetQuestionnairesMMAS8Failure,
  GetQuestionnairesPatientSCCAISuccess,
  GetQuestionnairesPatientSCCAIFailure,
  GetQuestionnairesPHQ9,
  GetQuestionnairesPHQ9Success,
  GetQuestionnairesPHQ9Failure,
  GetQuestionnairesPSQI,
  GetQuestionnairesPSQISuccess,
  GetQuestionnairesIPAQSFFailure,
  GetQuestionnairesPSQIFailure,
  GetQuestionnairesMIAHCD,
  GetQuestionnairesMIAHUCSuccess,
  GetQuestionnairesPRISM, GetQuestionnairesPRISMFailure, GetQuestionnairesPRISMSuccess
} from '../actions/doctor.actions';
import {DoctorService} from '../../services/doctor.service';
import {
  QuestionnaireClinicalSCCAIResult,
  QuestionnaireEORTCResult,
  QuestionnaireEQ5D5LResult,
  QuestionnaireHADSResult,
  QuestionnaireHBIResult,
  QuestionnaireIBDDiskResult,
  QuestionnaireIPAQSFResult,
  QuestionnaireMIAHCDResult,
  QuestionnaireMIAHUCResult,
  QuestionnaireMMAS8Result,
  QuestionnairePatientSCCAIResult, QuestionnairePHQ9Result, QuestionnairePRISMResult, QuestionnairePSQIResult,
  QuestionnaireSIBDQResult,
  QuestionnaireTSQMResult,
  QuestionnaireWPAIUCResult
} from '../../model/questionnaires/questionnaire-results';
import {Patient} from '../../model/user/Patient';
import {JobModel} from '../../model/job/JobModel';

@Injectable()
export class DoctorEffects {

  constructor(
    private actions$: Actions,
    private doctorService: DoctorService) {}

  // effects go here

  @Effect()
  GetPatients: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_PATIENTS),
    tap( () => console.log(DoctorActionTypes.GET_PATIENTS + ' Effect - get patients-control-panel')),
    switchMap(() => {
      return this.doctorService.getPatients()
        .map((patientArray) => {
          return new GetPatientsSuccess({
            patients: patientArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetPatientsFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetPatientsSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_PATIENTS_SUCCESS),
    tap((data) => {
      console.log(data);
      localStorage.setItem('patients', JSON.stringify(data.payload.patients));
    })
  );

  @Effect({ dispatch: false })
  GetPatientsFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_PATIENTS_FAILURE),
    tap((data) => {
      const empty: Patient[] = [];
      localStorage.setItem('patients', JSON.stringify(empty));
    })
  );

  // TODO: add new effect after a new questionnaire type added

  @Effect()
  GetQuestionnairesClinicalSCCAI: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_C_SCCAI),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_C_SCCAI + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesClinicalSCCAI) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesClinicalSCCAI(payload, 'CLINICAL-SCCAI')
        .map((sccaiArray: QuestionnaireClinicalSCCAIResult[]) => {
          return new GetQuestionnairesClinicalSCCAISuccess({
            resultsClinicalSCCAI: sccaiArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesClinicalSCCAIFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesClinicalSCCAISuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_C_SCCAI_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('clinical-sccai') !== null) {
        localStorage.removeItem('clinical-sccai');
      }
      localStorage.setItem('clinical-sccai', JSON.stringify(data.payload.resultsClinicalSCCAI));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesClinicalSCCAIFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_C_SCCAI_FAILURE)
  );

  @Effect()
  GetQuestionnairesEORTC: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_EORTC),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_EORTC + ' Effect - get questionnaires EORTC for patient')),
    map((action: GetQuestionnairesEORTC) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesEORTC(payload, 'EORTC-QLQ')
        .map((eortcArray: QuestionnaireEORTCResult[]) => {
          return new GetQuestionnairesEORTCSuccess({
            resultsEORTC: eortcArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesEORTCFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesEORTCSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_EORTC_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('eortc-qlq') !== null) {
        localStorage.removeItem('eortc-qlq');
      }
      localStorage.setItem('eortc-qlq', JSON.stringify(data.payload.resultsEORTC));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesEORTCFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_EORTC_FAILURE)
  );

  @Effect()
  GetQuestionnairesEQ5D5L: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_EQ5D5L),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_EQ5D5L + ' Effect - get questionnaires EQ5D5L for patient')),
    map((action: GetQuestionnairesEQ5D5L) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesEQ5D5L(payload, 'EQ5D5L')
        .map((eq5d5lArray: QuestionnaireEQ5D5LResult[]) => {
          return new GetQuestionnairesEQ5D5LSuccess({
            resultsEQ5D5L: eq5d5lArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesEQ5D5LFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesEQ5D5LSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_EQ5D5L_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('eq5d5l') !== null) {
        localStorage.removeItem('eq5d5l');
      }
      localStorage.setItem('eq5d5l', JSON.stringify(data.payload.resultsEQ5D5L));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesEQ5D5LFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_EQ5D5L_FAILURE)
  );

  @Effect()
  GetQuestionnairesHADS: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_HADS),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_HADS + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesHADS) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesHADS(payload, 'HADS')
        .map((hadsArray: QuestionnaireHADSResult[]) => {
          return new GetQuestionnairesHADSSuccess({
            resultsHADS: hadsArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesEORTCFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesHADSSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_HADS_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('hads') !== null) {
        localStorage.removeItem('hads');
      }
      localStorage.setItem('hads', JSON.stringify(data.payload.resultsHADS));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesHADSFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_HADS_FAILURE)
  );

  @Effect()
  GetQuestionnairesHBI: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_HBI),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_HBI + ' Effect - get questionnaires HBI for patient')),
    map((action: GetQuestionnairesHBI) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesHBI(payload, 'HBI')
        .map((hbiArray: QuestionnaireHBIResult[]) => {
          return new GetQuestionnairesHBISuccess({
            resultsHBI: hbiArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesHBIFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesHBISuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_HBI_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('hbi') !== null) {
        localStorage.removeItem('hbi');
      }
      localStorage.setItem('hbi', JSON.stringify(data.payload.resultsHBI));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesHBIFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_HBI_FAILURE)
  );

  @Effect()
  GetQuestionnairesIBDDisk: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_IBDDISK),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_IBDDISK + ' Effect - get questionnaires IBDDisk for patient')),
    map((action: GetQuestionnairesIBDDisk) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesIBDDisk(payload, 'IBD-DISK')
        .map((ibdDiskArray: QuestionnaireIBDDiskResult[]) => {
          return new GetQuestionnairesIBDDiskSuccess({
            resultsIBDDisk: ibdDiskArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesIBDDiskFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesIBDDiskSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_IBDDISK_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('ibd-disk') !== null) {
        localStorage.removeItem('ibd-disk');
      }
      localStorage.setItem('ibd-disk', JSON.stringify(data.payload.resultsIBDDisk));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesIBDDiskFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_IBDDISK_FAILURE)
  );

  @Effect()
  GetQuestionnairesIPAQSF: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_IPAQSF),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_IPAQSF + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesIPAQSF) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesIPAQSF(payload, 'IPAQ-SF')
        .map((ipaqSfArray: QuestionnaireIPAQSFResult[]) => {
          return new GetQuestionnairesIPAQSFSuccess({
            resultsIPAQSF: ipaqSfArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesIPAQSFFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesIPAQSFSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_IPAQSF_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('ipaq-sf') !== null) {
        localStorage.removeItem('ipaq-sf');
      }
      localStorage.setItem('ipaq-sf', JSON.stringify(data.payload.resultsIPAQSF));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesIPAQSFFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_IPAQSF_FAILURE)
  );

  @Effect()
  GetQuestionnairesMIAHCD: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_CD),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_CD + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesMIAHCD) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesMIAHCD(payload, 'MIAH-CD')
        .map((miahcdArray: QuestionnaireMIAHCDResult[]) => {
          return new GetQuestionnairesMIAHCDSuccess({
            resultsMIAHCD: miahcdArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesMIAHCDFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesMIAHCDSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_CD_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('miah-cd') !== null) {
        localStorage.removeItem('miah-cd');
      }
      localStorage.setItem('miah-cd', JSON.stringify(data.payload.resultsMIAHCD));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesMIAHCDFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_CD_FAILURE)
  );

  @Effect()
  GetQuestionnairesMIAHUC: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_UC),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_UC + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesMIAHUC) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesMIAHUC(payload, 'MIAH-UC')
        .map((miahucArray: QuestionnaireMIAHUCResult[]) => {
          return new GetQuestionnairesMIAHUCSuccess({
            resultsMIAHUC: miahucArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesMIAHUCFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesMIAHUCSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_UC_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('miah-uc') !== null) {
        localStorage.removeItem('miah-uc');
      }
      localStorage.setItem('miah-uc', JSON.stringify(data.payload.resultsMIAHUC));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesMIAHUCFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MIAH_UC_FAILURE)
  );

  @Effect()
  GetQuestionnairesMMAS8: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MMAS8),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_MMAS8 + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesMIAHUC) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesMMAS8(payload, 'MMAS8')
        .map((mmas8Array: QuestionnaireMMAS8Result[]) => {
          return new GetQuestionnairesMMAS8Success({
            resultsMMAS8: mmas8Array
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesMMAS8Failure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesMMAS8Success: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MMAS8_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('mmas8') !== null) {
        localStorage.removeItem('mmas8');
      }
      localStorage.setItem('mmas8', JSON.stringify(data.payload.resultsMMAS8));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesMMAS8Failure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_MMAS8_FAILURE)
  );

  @Effect()
  GetQuestionnairesPatientSCCAI: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_P_SCCAI),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_P_SCCAI + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesMIAHUC) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesPatientSCCAI(payload, 'PATIENT-SCCAI')
        .map((patientSCCAIArray: QuestionnairePatientSCCAIResult[]) => {
          return new GetQuestionnairesPatientSCCAISuccess({
            resultsPatientSCCAI: patientSCCAIArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesPatientSCCAIFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesPatientSCCAISuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_P_SCCAI_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('patient-sccai') !== null) {
        localStorage.removeItem('patient-sccai');
      }
      localStorage.setItem('patient-sccai', JSON.stringify(data.payload.resultsPatientSCCAI));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesPatientSCCAIFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_P_SCCAI_FAILURE)
  );

  @Effect()
  GetQuestionnairesPHQ9: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PHQ9),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_PHQ9 + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesPHQ9) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesPHQ9(payload, 'PHQ9')
        .map((phq9Array: QuestionnairePHQ9Result[]) => {
          return new GetQuestionnairesPHQ9Success({
            resultsPHQ9: phq9Array
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesPHQ9Failure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesPHQ9Success: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PHQ9_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('phq9') !== null) {
        localStorage.removeItem('phq9');
      }
      localStorage.setItem('phq9', JSON.stringify(data.payload.resultsPHQ9));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesPHQ9Failure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PHQ9_FAILURE)
  );

  @Effect()
  GetQuestionnairesPRISM: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PRISM),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_PRISM + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesPRISM) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesPRISM(payload, 'PRISM')
        .map((prismArray: QuestionnairePRISMResult[]) => {
          return new GetQuestionnairesPRISMSuccess({
            resultsPRISM: prismArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesPRISMFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesPRISMSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PRISM_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('prism') !== null) {
        localStorage.removeItem('prism');
      }
      localStorage.setItem('prism', JSON.stringify(data.payload.resultsPRISM));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesPRISMFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PRISM_FAILURE)
  );

  @Effect()
  GetQuestionnairesPSQI: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PSQI),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_PSQI + ' Effect - get questionnaires for patient')),
    map((action: GetQuestionnairesPSQI) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesPSQI(payload, 'PSQI')
        .map((psqiArray: QuestionnairePSQIResult[]) => {
          return new GetQuestionnairesPSQISuccess({
            resultsPSQI: psqiArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesPSQIFailure({error}));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesPSQISuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PSQI_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('psqi') !== null) {
        localStorage.removeItem('psqi');
      }
      localStorage.setItem('psqi', JSON.stringify(data.payload.resultsPSQI));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesPSQIFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_PSQI_FAILURE)
  );


  @Effect()
  GetQuestionnairesSIBDQ: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_SIBDQ),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_SIBDQ + ' Effect - get questionnaires SIBDQ for patient')),
    map((action: GetQuestionnairesSIBDQ) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesSIBDQ(payload, 'SIBDQ')
        .map((sibdqArray: QuestionnaireSIBDQResult[]) => {
          return new GetQuestionnairesSIBDQSuccess({
            resultsSIBDQ: sibdqArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesSIBDQFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesSIBDQSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_SIBDQ_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('sibdq') !== null) {
        localStorage.removeItem('sibdq');
      }
      localStorage.setItem('sibdq', JSON.stringify(data.payload.resultsSIBDQ));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesSIBDQFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_SIBDQ_FAILURE)
  );

  @Effect()
  GetQuestionnairesTSQM: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_TSQM),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_TSQM + ' Effect - get questionnaires TSQM for patient')),
    map((action: GetQuestionnairesTSQM) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesTSQM(payload, 'TSQM')
        .map((tsqmArray: QuestionnaireTSQMResult[]) => {
          return new GetQuestionnairesTSQMSuccess({
            resultsTSQM: tsqmArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesTSQMFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesTSQMSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_TSQM_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('tsqm') !== null) {
        localStorage.removeItem('tsqm');
      }
      localStorage.setItem('tsqm', JSON.stringify(data.payload.resultsTSQM));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesTSQMFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_TSQM_FAILURE)
  );

  @Effect()
  GetQuestionnairesWPAIUC: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_WPAIUC),
    tap( () => console.log(DoctorActionTypes.GET_QUESTIONNAIRES_WPAIUC + ' Effect - get questionnaires WPAI-UC for patient')),
    map((action: GetQuestionnairesWPAIUC) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getQuestionnairesWPAIUC(payload, 'WPAI-UC')
        .map((wpaiucArray: QuestionnaireWPAIUCResult[]) => {
          return new GetQuestionnairesWPAIUCSuccess({
            resultsWPAIUC: wpaiucArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesWPAIUCFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetQuestionnairesWPAIUCSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_WPAIUC_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('wpai-uc') !== null) {
        localStorage.removeItem('wpai-uc');
      }
      localStorage.setItem('wpai-uc', JSON.stringify(data.payload.resultsWPAIUC));
    })
  );

  @Effect({ dispatch: false })
  GetQuestionnairesWPAIUCFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_QUESTIONNAIRES_WPAIUC_FAILURE)
  );

  @Effect()
  GetJobs: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_JOBS),
    tap( () => console.log(DoctorActionTypes.GET_JOBS + ' Effect - get Jobs for patient')),
    map((action: GetJobs) => action.payload),
    switchMap((payload) => {
      return this.doctorService.getJobs(payload)
        .map((jobsArray: JobModel[]) => {
          return new GetJobsSuccess({
            jobs: jobsArray
          });
        })
        .catch((error) => {
          console.log(error);
          return Observable.of(new GetQuestionnairesWPAIUCFailure({ error }));
        });
    }));

  @Effect({ dispatch: false })
  GetJobsSuccess: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_JOBS_SUCCESS),
    tap((data) => {
      console.log(data);
      if (localStorage.getItem('jobs') !== null) {
        localStorage.removeItem('jobs');
      }
      localStorage.setItem('jobs', JSON.stringify(data.payload.jobs));
    })
  );

  @Effect({ dispatch: false })
  GetJobsFailure: Observable<any> = this.actions$.pipe(
    ofType(DoctorActionTypes.GET_JOBS_FAILURE)
  );

}
